@@ -13,7 +13,7 @@ from DateTime import DateTime
 
 from I18n import _
 import Permissions
-from Utils import get_transaction, BLATHER, formattedTraceback, DateTimeSyntaxError, callHooks
+from Utils import get_transaction, BLATHER, formattedTraceback, DateTimeSyntaxError, callHooks, safe_hasattr
 from pagetypes import PAGE_TYPE_UPGRADES
 from Defaults import PAGE_METADATA, \
      TEXTINDEXES, FIELDINDEXES, KEYWORDINDEXES, DATEINDEXES, PATHINDEXES
@@ -69,7 +69,7 @@ class PageAdminSupport:
             try:
                 p.upgrade(REQUEST)
                 p.upgradeId(REQUEST)
-                p.fixPageEncoding()
+                p.fixEncoding()
                 if render:
                     p.preRender(clear_cache=1)
                     msg = 'upgraded and pre-rendered page'
@@ -175,7 +175,7 @@ class PageAdminSupport:
         # NB be a bit careful about  acquisition while doing this
 
         # migrate a WikiForNow _st_data attribute
-        if hasattr(self.aq_base, '_st_data'):
+        if safe_hasattr(self.aq_base, '_st_data'):
             self.raw = self._st_data
             del self._st_data
             changed = 1
@@ -190,21 +190,31 @@ class PageAdminSupport:
 
         # Pre-0.9.10, creation_time has been a string in custom format and
         # last_edit_time has been a DateTime. Now both are kept as
-        # ISO-format strings. Might not be strictly necessary to upgrade
+        # ISO 8601-format strings. Might not be strictly necessary to upgrade
         # in all cases.. will cause a lot of bobobase_mod_time
         # updates.. do it anyway.
         if not self.last_edit_time:
-            self.last_edit_time = self.bobobase_modification_time().ISO()
+            self.last_edit_time = self.bobobase_modification_time().ISO8601()
             changed = 1
         elif type(self.last_edit_time) is not StringType:
-            self.last_edit_time = self.last_edit_time.ISO()
+            self.last_edit_time = self.last_edit_time.ISO8601()
             changed = 1
-        elif len(self.last_edit_time) != 19:
+        elif len(self.last_edit_time) != 25:
             try: 
-                self.last_edit_time = DateTime(self.last_edit_time).ISO()
+                if len(self.last_edit_time) == 19: # older "ISO()" format
+                    # we're using the behaviour that was standard in
+                    # Zope <= 2.9, where a timestamp without timezone
+                    # information was assumed to be in UTC (aka GMT)
+                    self.last_edit_time = \
+                    DateTime(self.last_edit_time+' GMT+0').ISO8601()
+                else:
+                    # some other timestamp format, leave tz information
+                    # untouched, or let DateTime guess at it
+                    self.last_edit_time = \
+                    DateTime(self.last_edit_time).ISO8601()
                 changed = 1
             except DateTimeSyntaxError:
-                # can't convert to ISO, just leave it be
+                # can't convert to ISO 8601, just leave it be
                 pass
 
         # If no creation_time, just leave it blank for now.
@@ -212,14 +222,19 @@ class PageAdminSupport:
         if not self.creation_time:
             pass
         elif type(self.creation_time) is not StringType:
-            self.creation_time = self.creation_time.toZone('UTC').ISO()
+            self.creation_time = self.creation_time.ISO8601()
             changed = 1
-        elif len(self.creation_time) != 19:
+        elif len(self.creation_time) != 25:
             try: 
-                self.creation_time = DateTime(self.creation_time).toZone('UTC').ISO()
+                if len(self.creation_time) == 19: # older "ISO()" format
+                    self.creation_time = \
+                    DateTime(self.creation_time+' GMT+0').ISO8601()
+                else:
+                    self.creation_time = \
+                    DateTime(self.creation_time).ISO8601()
                 changed = 1
             except DateTimeSyntaxError:
-                # can't convert to ISO, just leave it be
+                # can't convert to ISO 8601, just leave it be
                 pass
 
         # _wikilinks, _links and _prelinked are no longer used
@@ -228,7 +243,7 @@ class PageAdminSupport:
             '_links',
             '_prelinked',
             ):
-            if hasattr(self.aq_base,a): 
+            if safe_hasattr(self.aq_base,a): 
                 delattr(self,a)
                 self.clearCache()
                 changed = 1 
@@ -285,9 +300,13 @@ class PageAdminSupport:
         # PageMailSupport does a bit more (merge here ?)
         self._upgradeSubscribers()
 
-        # also make sure there is an up-to-date outline cache
-        self.wikiOutline()
+        # make sure there is a catalog for this wiki
+        self.ensureCatalog()
 
+        # make sure there is an up-to-date outline cache
+        self.ensureWikiOutline()
+
+    #XXX this is copied code which needs to be refactored after 0.60
     security.declareProtected('Manage properties', 'setupPages')
     def setupPages(self,REQUEST=None):
         """
@@ -309,11 +328,13 @@ class PageAdminSupport:
                         if m.group(2): parents = split(strip(m.group(2)),',')
                         else: parents = []
                         text = m.group(3)
+                        # XXX: this will choke if it tries to
+                        # create a page with a .png file
                         self.create(name,text=text)
         if REQUEST:
             REQUEST.RESPONSE.redirect(self.pageUrl())
 
-    #XXX does this not attempt to create everything ?
+    #XXX this is copied code which needs to be refactored after 0.60
     security.declareProtected('Manage properties', 'setupPages')
     def setupDtmlPages(self,REQUEST=None):
         """
@@ -326,17 +347,20 @@ class PageAdminSupport:
         dir = os.path.join(package_home(globals()),'skins','zwiki')
         filenames = os.listdir(dir)
         for filename in filenames:
-            m = re.search(r'(.+)\.(.+)',filename)
-            if m:
-                name, type = m.group(1), m.group(2)
-                if not self.pageWithName(name):
-                    text=open(os.path.join(dir,filename),'r').read()
-                    # parse optional parents list
-                    m = re.match(r'(?si)(^#parents:(.*?)\n)?(.*)',text)
-                    if m.group(2): parents = split(strip(m.group(2)),',')
-                    else: parents = []
-                    text = m.group(3)
-                    self.create(name,text=text)
+            if filename[-5:] != '.dtml':
+                pass
+            else:
+                m = re.search(r'(.+)\.(.+)',filename)
+                if m:
+                    name, type = m.group(1), m.group(2)
+                    if not self.pageWithName(name):
+                        text=open(os.path.join(dir,filename),'r').read()
+                        # parse optional parents list
+                        m = re.match(r'(?si)(^#parents:(.*?)\n)?(.*)',text)
+                        if m.group(2): parents = split(strip(m.group(2)),',')
+                        else: parents = []
+                        text = m.group(3)
+                        self.create(name,text=text)
         if REQUEST:
             REQUEST.RESPONSE.redirect(self.pageUrl())
 
@@ -390,10 +414,9 @@ class PageAdminSupport:
         Safe to call more than once; will ignore any already existing
         items. For simplicity we install all metadata for plugins (like
         Tracker) here as well.
-        """                
-        #XXX during unit testing, somehow a non-None catalog is false
-        #if not self.catalog():
-        if self.catalogId() == 'NONE':
+        """
+        if self.inRevisionsFolder(): return
+        if not self.hasCatalog():
             self.folder().manage_addProduct['ZCatalog'].manage_addZCatalog('Catalog','')
         catalog = self.catalog()
         catalogindexes, catalogmetadata = catalog.indexes(), catalog.schema()
@@ -426,7 +449,18 @@ class PageAdminSupport:
         if REQUEST:
             REQUEST.RESPONSE.redirect(self.pageUrl())
             
-    def fixPageEncoding(self, FROM='iso8859-1', TO='utf-8'):
+    def ensureCatalog(self):
+        """
+        Ensure this wiki has a zcatalog, for fast standardized searching.
+
+        We'll create one if needed, and index all current pages. This
+        could take a while (minutes) in a large wiki.
+        """
+        if not self.hasCatalog() and not self.inRevisionsFolder():
+            BLATHER('creating catalog for wiki',self.folder().getId())
+            self.setupCatalog()
+
+    def fixEncoding(self, FROM='iso8859-1', TO='utf-8', REQUEST=None):
         """
         Try to fix any character encoding problems in this page's name or text.
 
@@ -447,15 +481,15 @@ class PageAdminSupport:
             #       title=unicode(self.pageName(), FROM).encode(TO),
             #       log=log)
             # this won't
-            self.setText(unicode(self.text(), FROM).encode(TO), REQUEST=REQUEST)
+            self.setText(unicode(self.text(), FROM).encode(TO))
             # this will (if the name really needs fixing)
-            self.rename(unicode(self.pageName(), FROM).encode(TO), sendmail=0, REQUEST=REQUEST)
+            self.rename(unicode(self.pageName(), FROM).encode(TO))
             return True
+        if REQUEST: REQUEST.RESPONSE.redirect(self.pageUrl())
 
-    def fixAllPagesEncoding(self):
-        for p in self.pageObjects():
-            fixPageEncoding(p)
-        self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1'])
+    def fixAllPagesEncoding(self, REQUEST=None):
+        for p in self.pageObjects(): fixEncoding(p)
+        if REQUEST: REQUEST.RESPONSE.redirect(REQUEST['URL1'])
 
 InitializeClass(PageAdminSupport)
 
