@@ -1,18 +1,23 @@
-# CMF compatibility
-# this is split between CMF.py, CMFInit.py and Extensions/cmf_install_zwiki.py
-# (unfortunately) and it's still not right (need to import makeWikiPage
-# for PUT_factory ?). 
-# move CMFInstall into the skin ?
+# CMF/Plone compatibility
+# See also __init__.py and Extensions/Install.py
 
-from types import *
-import string, re, os
+#from types import *
 from Globals import InitializeClass
+from Utils import safe_hasattr
 
 try:
     from Products import CMFCore
-    HAS_CMF = 1
+    CMFCore = CMFCore # pyflakes
+    HAS_CMF = True
 except ImportError:
-    HAS_CMF = 0
+    HAS_CMF = False
+
+try:
+    from Products import CMFPlone
+    CMFPlone = CMFPlone # pyflakes
+    HAS_PLONE = True
+except ImportError:
+    HAS_PLONE = False
 
 # XXX also need to skip when running unit tests ?
 if not HAS_CMF:    
@@ -20,16 +25,14 @@ if not HAS_CMF:
         __implements__ = ()
         def supportsCMF(self): return 0
         def inCMF(self): return 0
+        def inPlone(self): return 0
 
 else:
     from AccessControl import ClassSecurityInfo
-    from Acquisition import aq_base, aq_inner, aq_parent
     from OFS.DTMLDocument import DTMLDocument
     from Products.CMFCore.PortalContent import PortalContent
     from Products.CMFDefault.SkinnedFolder import SkinnedFolder
-    from Products import CMFDefault
     from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
-    from AccessControl import getSecurityManager, ClassSecurityInfo
     from DateTime import DateTime
     import Permissions
     from Defaults import PAGE_PORTALTYPE
@@ -56,9 +59,9 @@ else:
         security.declarePrivate('setModificationDate')
         def setModificationDate(self, modification_date=None):
             if modification_date is None:
-                self.last_edit_time = DateTime().toZone('UTC').ISO()
+                self.last_edit_time = DateTime().ISO8601()
             else:
-                self.last_edit_time = self._datify(modification_date).toZone('UTC').ISO()
+                self.last_edit_time = self._datify(modification_date).ISO8601()
 
         security.declarePublic('modified')
         def modified(self):
@@ -74,7 +77,7 @@ else:
 
         security.declarePublic( 'CreationDate' )
         def CreationDate(self):
-            return self.creationTime().ISO()
+            return self.creationTime().ISO8601()
 
         security.declarePublic( 'getPageTitle' )
         def getPageTitle(self, here=None, template=None, portal_title=None):
@@ -117,8 +120,13 @@ else:
 
         security.declarePublic('inCMF')
         def inCMF(self):
-            """return true if this page is in a CMF portal"""
-            return hasattr(self.aq_inner.aq_parent,'portal_membership')
+            """return true if this page is in a CMF site"""
+            return safe_hasattr(self.aq_inner.aq_parent,'portal_membership')
+
+        security.declarePublic('inPlone')
+        def inPlone(self):
+            """return true if this page is in a Plone site"""
+            return self.inCMF() and safe_hasattr(self.aq_inner.aq_parent,'plone_utils')
 
         def __init__(self, source_string='', mapping=None, __name__=''):
             DTMLDocument.__init__(self,
@@ -185,5 +193,6 @@ else:
             """
             return self == self.defaultPage()
 
+            
 InitializeClass(PageCMFSupport)
 
