@@ -1,13 +1,11 @@
 # catalog awareness for zwiki pages
 # originally based on Casey Duncan's DTMLDocumentExt 0.1
 
-import string
-
 from AccessControl import getSecurityManager, ClassSecurityInfo
 from Globals import InitializeClass
 
 import Permissions
-from Utils import BLATHER,formattedTraceback
+from Utils import BLATHER,formattedTraceback,safe_hasattr
 
 
 class PageCatalogSupport:
@@ -34,13 +32,13 @@ class PageCatalogSupport:
 
         By default, Zwiki looks for a catalog named 'Catalog' in this
         folder (will not acquire) or a 'portal_catalog' in this folder
-        or above (will acquire).
+        or above (will acquire). Revision objects never have a catalog.
         """
+        if self.inRevisionsFolder(): return None
         folder = self.folder()
-        #if not folder: return None ?
         folderaqbase = getattr(folder,'aq_base',
                                folder) # make tests work
-        if hasattr(folderaqbase,'Catalog') and hasattr(folderaqbase.Catalog,'indexes'):
+        if safe_hasattr(folderaqbase,'Catalog') and safe_hasattr(folderaqbase.Catalog,'indexes'):
             return folder.Catalog
         else:
             try:
@@ -75,7 +73,8 @@ class PageCatalogSupport:
         (['index1','index2'],['metadata1'])
         """
         catalog = self.catalog()
-        if not catalog: return 0
+        #if not catalog: return 0  #see #1349
+        if self.catalogId()=='NONE': return 0
         catalogindexes, catalogmetadata = catalog.indexes(), catalog.schema()
         indexes, metadata = indexesAndMetadata
         for i in indexes:
@@ -93,19 +92,17 @@ class PageCatalogSupport:
 
     def url(self):
         """Return the absolute object path"""
-        return string.join(self.getPhysicalPath(),'/')
+        return '/'.join(self.getPhysicalPath())
 
     getPath = url
 
     security.declareProtected(Permissions.View, 'index_object')
     def index_object(self,idxs=[],log=1):
-        """Index this page in the wiki's catalog, if any, and log problems.
-
-        Updates only certain indexes, if specified. 
+        """Index this page in the wiki's catalog, if any, and log
+        problems.  Updates only certain indexes, if specified.
         """
         if self.hasCatalog() and self.isCatalogable():
-            if log:
-                BLATHER('indexing',self.url())
+            if log: BLATHER('indexing',self.url())
             try:
                 self.catalog().catalog_object(self,self.url(),idxs)
             except:
