@@ -1,4 +1,3 @@
-import string
 from testsupport import *
 ZopeTestCase.installProduct('ZCatalog')
 ZopeTestCase.installProduct('ZWiki')
@@ -18,10 +17,3 @@ class Tests(ZwikiTestCase):
         p.comment('test')
         self.assertEqual(p.commentCount(),2)
 
-    def test_unicodeInComments(self):
-        p = self.page
-        u = unicode(string.uppercase,'utf8')
-        p.comment(text=u)
-        self.assertEqual(p.commentCount(),1)
-        self.assertEqual(p.comments()[-1].get_payload(),u)
-
