@@ -17,11 +17,16 @@ LINK_TO_ALL_OBJECTS = 0      # link to non-wiki page objects ? unimplemented
 LARGE_FILE_SIZE = 1024*1024  # images larger than this will not be inlined
 LEAVE_PLACEHOLDER = 0        # leave a placeholder page when renaming ?
 WIKINAME_LINKS = 1           # enable/disable various wiki link syntaxes 
-BRACKET_LINKS = 1            # by default
+ISSUE_LINKS = 1              # by default
+BRACKET_LINKS = 1
 DOUBLE_BRACKET_LINKS = 1
 DOUBLE_PARENTHESIS_LINKS = 0
 BORING_PAGES = ['TestPage','SandBox'] # pages we don't want to see/hear much
 IDS_TO_AVOID = ['RESPONSE','REQUEST','Epoz','epoz','URL','outline','recycle_bin']
+CONDITIONAL_HTTP_GET = 0     # handle If-modified-since headers with 304 responses
+CONDITIONAL_HTTP_GET_IGNORE = [ 'allow_dtml' ] 
+                             # ignore pages with these properties set to 
+                             # non-False values
 
 # Standard metadata fields which Zwiki will expect in page brain objects.
 # Plugins will add more of these.
