@@ -9,6 +9,7 @@ from urllib import quote, unquote
 from types import *
 from email.Message import Message
 from copy import deepcopy
+import os.path
 
 import ZODB # need this for pychecker
 from AccessControl import getSecurityManager, ClassSecurityInfo
@@ -16,19 +17,25 @@ from App.Common import rfc1123_date
 from DateTime import DateTime
 from Globals import InitializeClass
 try:
-    from zope.app.content_types import guess_content_type
+    from zope.contenttype import guess_content_type
 except ImportError:
-    from OFS.content_types import guess_content_type
+    try:
+        from zope.app.content_types import guess_content_type
+    except ImportError:
+        from OFS.content_types import guess_content_type
 from OFS.DTMLDocument import DTMLDocument
 from OFS.ObjectManager import BadRequestException
+from OFS.ObjectManager import checkValidId
+from zExceptions import BadRequest
 import OFS.Image
 
 from pagetypes import PAGETYPES
 from Defaults import DISABLE_JAVASCRIPT, LARGE_FILE_SIZE, LEAVE_PLACEHOLDER
 import Permissions
 from Regexps import javascriptexpr, htmlheaderexpr, htmlfooterexpr
-from Utils import get_transaction, BLATHER, parseHeadersBody
+from Utils import get_transaction, BLATHER, parseHeadersBody, safe_hasattr
 from I18n import _
+from Diff import addedtext, textdiff
 
 
 class PageEditingSupport:
@@ -37,35 +44,48 @@ class PageEditingSupport:
     def checkPermission(self, permission, object):
         return getSecurityManager().checkPermission(permission,object)
 
-    security.declareProtected(Permissions.Add, 'create') 
+    security.declarePublic('createform')      # check permissions at runtime
     def create(self,page=None,text='',type=None,title='',REQUEST=None,log='',
                sendmail=1, parents=None, subtopics=None, pagename=None):
-        """
-        Create a new wiki page, with optional extras.
+        """Create a new wiki page, and optionally do extra stuff. Normally
+        called via edit().
 
-        Normally edit() will call this for you. 
+        The page name arguments are confusing. Here's the situation as I
+        understand it: XXX cleanup
+
+        - page:     the "original" name of the page we are to create.
+        - pagename: an alternate spelling of the above, to ease page 
+                    management form implementation. Pass one or the other.
+        - title:    optional "new" name to rename to after creation.
+                    This allows us to handle the zwiki and CMF/plone
+                    edit forms smoothly, supporting rename during creation.
+
+        Page names are assumed to come url-quoted. For the new page id, we use
+        a url-safe canonical id derived from the name. (This name and id are
+        assumed to be available, or Zwiki would not be calling create.)
+
+        Other arguments:
+
+        - text:      initial content for the new page
+        - type:      id of the page type to use (rst, html...)
+        - REQUEST:   standard zope argument, pass this to preserve any
+                     user authentication. Also, may include file upload
+                     data in which case the file is uploaded to the wiki.
+        - log:       optional note for edit history and mail-out subject
+        - sendmail:  sends mail-out to wiki subscribers, unless disabled
+        - parents:   the names of the new page's parent(s), if any
+        - subtopics: if non-None, sets the subtopics display property
 
-        We assume the page name comes url-quoted. If it's not a url-safe
-        name, we will create the page with a similar url-safe id, which we
-        assume won't match any existing page (or zwiki would have linked
-        instead of offering to create). Also it allows the alternate pagename
-        argument, to support the page management form (XXX temporary).
         Other features:
 
-        - can upload a file at the same time.  
-
-        - can set the subtopics display property
-
-        - can handle a rename during page creation. This helps CMF/Plone
-        and is occasionally useful.
-        
-        - checks the edit_needs_username property as well as permissions.
-
-        - redirects to the new page or to the denied view if appropriate
-
-        - returns the new page's name or None
-
+        - checks the edit_needs_username property as well as permissions
+        - redirects to the new page, or to the denied view
+        - returns the new page's name, or None
+        - if old revisions with this name exist, takes the next revision number
         """
+        if not self.checkPermission(Permissions.Add, self.folder()):
+            raise 'Unauthorized', (
+                _('You are not authorized to add pages in this wiki.'))
         if not self.checkSufficientId(REQUEST):
             if REQUEST: REQUEST.RESPONSE.redirect(self.pageUrl()+'/denied')
             return None
@@ -80,11 +100,14 @@ class PageEditingSupport:
         p = self.__class__(__name__=id)
         # set title now since manage_afterAdd will use it for wiki outline
         p.title = name
-        # newid should be the same as id, but don't assume
+        # place it in the folder
+        # newid should be the same as id, but we won't assume
         newid = self.folder()._setObject(id,p)
         p = getattr(self.folder(),newid)
-
+        # check and abort if text is spammy (after name is set up, for logging)
         p.checkForSpam(text)
+
+        p.ensureMyRevisionNumberIsLatest()
         p.setCreator(REQUEST)
         p.setLastEditor(REQUEST)
         p.setLastLog(log)
@@ -99,7 +122,7 @@ class PageEditingSupport:
         p.handleFileUpload(REQUEST)
         p.handleSubtopicsProperty(subtopics,REQUEST)
         if p.autoSubscriptionEnabled(): p.subscribeThisUser(REQUEST)
-        #if p.usingRegulations(): p.setRegulations(REQUEST,new=1)
+
         # users can alter the page name in the creation form; allow that.
         # We do a full rename, only after all the above, to make sure
         # everything gets handled properly.  We must first commit though,
@@ -133,7 +156,7 @@ class PageEditingSupport:
 
     security.declarePublic('isDavLocked')
     def isDavLocked(self):
-        return hasattr(self,'wl_isLocked') and self.wl_isLocked()
+        return safe_hasattr(self,'wl_isLocked') and self.wl_isLocked()
 
     security.declareProtected(Permissions.Comment, 'comment')
     def comment(self, text='', username='', time='',
@@ -191,6 +214,7 @@ class PageEditingSupport:
         # it to both source and _prerendered cached without re-rendering
         # the whole thing! This might not be legal for all future page
         # types.
+        self.saveRevision()
         # add to source, in standard rfc2822 format:
         t = str(m)
         self.checkForSpam(t)
@@ -260,7 +284,6 @@ class PageEditingSupport:
         attributes if a change is made
 
         - If the text begins with "DeleteMe", delete this page
-        (move it to the /recycle_bin)
 
         - If file has been submitted in REQUEST, create a file or
         image object and link or inline it on the current page.
@@ -302,13 +325,12 @@ class PageEditingSupport:
         # ok, changing p. We may do several things at once; each of these
         # handlers checks permissions and does the necessary.
         if p.handleDeleteMe(text,REQUEST,log): return
+        p.saveRevision()
         p.handleEditPageType(type,REQUEST,log)
         if text != None: p.handleEditText(text,REQUEST,subjectSuffix,log)
         p.handleSubtopicsProperty(subtopics,REQUEST)
         p.handleFileUpload(REQUEST,log)
         p.handleRename(title,leaveplaceholder,updatebacklinks,REQUEST,log)
-        #if self.usingRegulations(): p.handleSetRegulations(REQUEST)
-
         p.index_object()
 
         # tell browser to reload the page (or redirect elsewhere)
@@ -319,6 +341,10 @@ class PageEditingSupport:
                      REQUEST['URL2']+'/'+ quote(p.id())))
             except KeyError: pass
 
+    # This alternate spelling exists so that we can define an "edit" alias
+    # in Plone 3, needed to work around a createObject bug
+    update = edit
+
     def handleSubtopicsProperty(self,subtopics,REQUEST=None):
         if subtopics is not None:
             # do we have permission ?
@@ -328,17 +354,6 @@ class PageEditingSupport:
             subtopics = int(subtopics or '0')
             self.setSubtopicsPropertyStatus(subtopics,REQUEST)
 
-    # see Regulations.py
-    #def handleSetRegulations(self,REQUEST):
-    #    if REQUEST.get('who_owns_subs',None) != None:
-    #        # do we have permission ?
-    #        if not self.checkPermission(Permissions.ChangeRegs,self):
-    #            raise 'Unauthorized', (
-    #              _("You are not authorized to set this ZWiki Page's regulations."))
-    #        self.setRegulations(REQUEST)
-    #        self.preRender(clear_cache=1)
-    #        #self.setLastEditor(REQUEST)
-
     def handleEditPageType(self,type,REQUEST=None,log=''):
         # is the new page type valid and different ?
         if (type is not None and
@@ -357,10 +372,9 @@ class PageEditingSupport:
             self.setLastEditor(REQUEST)
             self.setLastLog(log)
 
-        """
-        Note log message, if provided.
-        """
+    security.declarePrivate('setLastLog')
     def setLastLog(self,log):
+        """Save an edit log message, if provided."""
         if log and string.strip(log):
             log = string.strip(log)
             get_transaction().note('"%s"' % log)
@@ -368,6 +382,9 @@ class PageEditingSupport:
         else:
             self.last_log = ''
 
+    def lastLog(self):
+        """Accessor for this page's last edit log message."""
+        return self.last_log
 
     def handleEditText(self,text,REQUEST=None, subjectSuffix='', log=''):
         old = self.read()
@@ -386,7 +403,7 @@ class PageEditingSupport:
             # Tries to count the links added by this edit. Not perfect -
             # existing links on a line that you tweak will be counted.
             # Not sure what happens if you replace existing links.
-            self.checkForSpam(self.addedText(old, new))
+            self.checkForSpam(addedtext(old, new))
                 
             # change it
             self.setText(text,REQUEST)
@@ -395,38 +412,10 @@ class PageEditingSupport:
 
             # send mail if appropriate
             self.sendMailToEditSubscribers(
-                self.textDiff(a=old,b=self.read()),
+                textdiff(a=old,b=self.read()),
                 REQUEST=REQUEST,
                 subject='(edit) %s' % log)
 
-    def handleDeleteMe(self,text,REQUEST=None,log=''):
-        if not text or not re.match('(?m)^DeleteMe', text):
-            return 0
-        if (not
-            (self.checkPermission(Permissions.Edit, self) or
-             (self.checkPermission(Permissions.Comment, self)
-              and find(self.cleanupText(text),self.read()) == 0))):
-            raise 'Unauthorized', (
-                _('You are not authorized to edit this ZWiki Page.'))
-        if not self.checkPermission(Permissions.Delete, self):
-            raise 'Unauthorized', (
-                _('You are not authorized to delete this ZWiki Page.'))
-        self.setLastLog(log)
-        self.recycle(REQUEST)
-
-        if REQUEST:
-            # redirect to first existing parent, or front page
-            destpage = ''
-            for p in self.parents:
-                if hasattr(self.folder(),p):
-                    destpage = p
-                    break
-            REQUEST.RESPONSE.redirect(self.wiki_url()+'/'+quote(destpage))
-            # I used to think redirect did not return, guess I was wrong
-
-        # return true to terminate edit processing
-        return 1
-
     def handleRename(self,newname,leaveplaceholder,updatebacklinks,
                       REQUEST=None,log=''):
         # rename does everything we need
@@ -435,56 +424,157 @@ class PageEditingSupport:
                            updatebacklinks=updatebacklinks,
                            REQUEST=REQUEST)
 
+    def handleDeleteMe(self,text,REQUEST=None,log=''):
+        if not text or not re.match('(?m)^DeleteMe', text):
+            return 0
+        if not self.checkPermission(Permissions.Edit, self):
+            raise 'Unauthorized', (
+                _('You are not authorized to edit this ZWiki Page.'))
+        if not self.checkPermission(Permissions.Delete, self):
+            raise 'Unauthorized', (
+                _('You are not authorized to delete this ZWiki Page.'))
+        self.setLastLog(log)
+        self.delete(REQUEST=REQUEST)
+        return 1 # terminate edit processing
+
     security.declareProtected(Permissions.Delete, 'delete')
     def delete(self,REQUEST=None, updatebacklinks=1, pagename=None):
         """
-        Delete (move to recycle_bin) this page, if permissions allow.
+        Delete this page, if permissions allow.
+
+        The last revision will be archived.
 
         If the pagename argument is provided (so named due to the page
         management form), we will try to redirect all links which point to
         this page, to that one. This is like doing a rename except this
         page vanishes in a puff of smoke. See also rename. As with that
         method, an updatebacklinks=0 argument will disable this.
-
         XXX no it won't ? also, need a flag to disable reparenting
         """
-        oldname, oldid = self.pageName(), self.getId()
-        # update parents attributes to avoid orphans
+        oldname,oldid,redirecturl = self.pageName(),self.getId(),self.upUrl()
+        # reparent my children, if any
         self.moveMyChildrenTo(self.primaryParentName())
-        # if a replacement page is specified, redirect all our backlinks there
+        # if a replacement pagename is specified, redirect my backlinks there
         if pagename and string.strip(pagename):
             self.replaceLinksThroughoutWiki(oldname,pagename,REQUEST)
-        # get parent url while we still can
-        redirecturl = self.upUrl()
-        # unindex (and remove from outline) and move to the recycle bin folder
-        self.recycle(REQUEST)
+        # archive my last revision
+        self.saveRevision()
+        # delete me - hopefully this updates outline and catalog too
+        self.folder().manage_delObjects([self.getId()])
         # notify subscribers if appropriate
         self.sendMailToEditSubscribers(
             'This page was deleted.\n',
             REQUEST=REQUEST,
             subjectSuffix='',
             subject='(deleted)')
+        # redirect somewhere sensible
         if REQUEST: REQUEST.RESPONSE.redirect(redirecturl)
 
-    def ensureRecycleBin(self):
-        if not hasattr(self.folder().aq_base,'recycle_bin'):
-            self.folder().manage_addFolder('recycle_bin', 'deleted wiki pages')
-
-    def recycle(self, REQUEST=None):
+    security.declareProtected(Permissions.Edit, 'revert')
+    def revert(self, rev, REQUEST=None):
         """
-        Move this page to the recycle_bin subfolder, creating it if necessary.
-        """
-        self.ensureRecycleBin()
-        f = self.folder()
-        # cut or paste also unindexes, I believe
-        cb = f.manage_cutObjects(self.getId())
-        # kludge! don't let manage_pasteObjects catalog the new location
-        # (or add it to wiki outline)
-        save = self.__class__.manage_afterAdd
-        self.__class__.manage_afterAdd = lambda self,item,container: None
-        f.recycle_bin.manage_pasteObjects(cb)
-        self.__class__.manage_afterAdd = save
+        Revert this page to the specified revision's state, updating history.
 
+        We do this by looking at the old page revision object and applying
+        a corrective edit to the current page. This records a new page
+        revision, reverts text, reverts parents, sends a mailout, etc.
+        Last editor, last editor ip, last edit time are updated, cf #1157.
+
+        Page renames are not reverted since the new revisions
+        implementation, stand by.
+        """
+        if not self.checkSufficientId(REQUEST):
+            return self.denied(
+                _("Sorry, this wiki doesn't allow anonymous edits. Please configure a username in options first."))
+        if not rev: return
+        rev = int(rev)
+        old = self.revision(rev)
+        if not old: return
+        reparent = self.getParents() != old.getParents()
+        if reparent and not self.checkPermission(Permissions.Reparent, self):
+            raise 'Unauthorized', (
+                _('You are not authorized to reparent this ZWiki Page.'))
+        #rename = self.pageName() != old.pageName():
+        #if rename and not self.checkPermission(Permissions.Rename, self):
+        #    raise 'Unauthorized', (
+        #        _('You are not authorized to rename this ZWiki Page.'))
+
+        self.saveRevision()
+        self.setText(old.text())
+        self.setPageType(old.pageTypeId())
+        self.setVotes(old.votes())
+        if reparent:
+            self.setParents(old.getParents())
+            self.updateWikiOutline()
+        #if rename:
+        #    self.rename(old.pageName())
+        self.setLastEditor(REQUEST)
+#         self.last_editor    = old.last_editor
+#         self.last_editor_ip = old.last_editor_ip
+#         self.last_edit_time = old.last_edit_time
+        self.setLastLog('reverted by %s' % self.usernameFrom(REQUEST))
+        self.index_object()
+        self.sendMailToEditSubscribers(
+            'This page was reverted to the %s version.\n' % old.last_edit_time,
+            REQUEST=REQUEST,
+            subjectSuffix='',
+            subject='(reverted)')
+        if REQUEST is not None:
+            REQUEST.RESPONSE.redirect(self.pageUrl())
+
+    security.declareProtected(Permissions.manage_properties, 'expunge')
+    def expunge(self, rev, REQUEST=None):
+        """
+        Revert this page to the specified revision, discarding later history.
+        """
+        if not rev: return
+        rev = int(rev)
+        savedrevs = self.revisionNumbers()[:-1]
+        if not rev in savedrevs: return
+        id = self.getId()
+        old = self.revision(rev)
+        old._setId(id)
+        self.folder()._delObject(id)
+        self.folder()._setObject(id,old)
+        # delete history
+        rf = self.revisionsFolder()
+        deadrevs = savedrevs[savedrevs.index(rev):]
+        for r in deadrevs:
+            rf._delObject('%s.%d' % (id,r))
+        BLATHER('expunged %s history after revision %d' % (id,rev))
+        if REQUEST is not None:
+            REQUEST.RESPONSE.redirect(self.pageUrl())
+
+    security.declareProtected(Permissions.manage_properties, 'expungeEditsBy')
+    def expungeEditsBy(self, username, REQUEST=None):
+        """Expunge all the recent edits to this page by username, if any."""
+        self.expunge(self.revisionNumberBefore(username), REQUEST=REQUEST)
+
+    security.declareProtected(Permissions.manage_properties, 'expungeEditsEverywhereBy')
+    def expungeEditsEverywhereBy(self, username, REQUEST=None, batch=0):
+        """
+        Expunge all the most recent edits by username throughout the wiki.
+
+        This is a powerful spam repair tool for managers. It removes all
+        recent consecutive edits by username from each page in the
+        wiki. The corresponding revisions will disappear from the page
+        history.  See #1157.
+        """
+        batch = int(batch)
+        n = 0
+        for p in self.pageObjects():
+            if p.last_editor == username:
+                n += 1
+                try:
+                    p.expungeEditsBy(username,REQUEST=REQUEST)
+                except IndexError:
+                    # IndexError - we don't have a version that old
+                    BLATHER('failed to expunge edits by %s at %s: %s' \
+                            % (username,p.id(),formattedTraceback()))
+                if batch and n % batch == 0:
+                    BLATHER('committing after %d expunges' % n)
+                    get_transaction().commit()
+        
     security.declareProtected(Permissions.Rename, 'rename')
     def rename(self,pagename,
                leaveplaceholder=LEAVE_PLACEHOLDER,
@@ -585,6 +675,7 @@ class PageEditingSupport:
         self.folder().manage_renameObject(self.getId(),newid)
         self.creation_time, self.creator, self.creator_ip = \
           creation_time, creator, creator_ip
+        self.ensureMyRevisionNumberIsLatest()
 
     def moveMyChildrenTo(self,newparent):
         children = self.childrenIdsAsList()
@@ -641,12 +732,12 @@ class PageEditingSupport:
     def handleFileUpload(self,REQUEST,log=''):
         # is there a file upload ?
         if (REQUEST and
-            hasattr(REQUEST,'file') and
-            hasattr(REQUEST.file,'filename') and
+            safe_hasattr(REQUEST,'file') and
+            safe_hasattr(REQUEST.file,'filename') and
             REQUEST.file.filename):     # XXX do something
 
             # figure out the upload destination
-            if hasattr(self,'uploads'):
+            if safe_hasattr(self,'uploads'):
                 uploaddir = self.uploads
             else:
                 uploaddir = self.folder()
@@ -680,6 +771,12 @@ class PageEditingSupport:
                 self._addFileLink(file_id, content_type, size, REQUEST)
                 self.setLastLog(log)
                 self.index_object()
+                self.sendMailToEditSubscribers(
+                    'Uploaded file "%s" on page "%s".\n' %
+                                        (file_id,self.pageName()),
+                    REQUEST=REQUEST,
+                    subjectSuffix='',
+                    subject='(renamed)')
             else:
                 # failed to create - give up (what about an error)
                 pass
@@ -693,7 +790,7 @@ class PageEditingSupport:
         size, or (None,None,None).
         """
         # macro to check folder contents without acquiring
-        folderHas = lambda folder,id: hasattr(folder.aq_base,id)
+        folderHas = lambda folder,id: safe_hasattr(folder.aq_base,id)
 
         # set id & title from filename
         title=str(title)
@@ -708,6 +805,15 @@ class PageEditingSupport:
             folder.uploads.isPrincipiaFolderish):
             folder = folder.uploads
 
+        # make sure we have an acceptable name,
+        # but do not insist on unique names:
+        try:
+            checkValidId(folder,id, allow_dup=1)
+        except BadRequest:
+            id, ext = os.path.splitext(id)
+            id = self.canonicalIdFrom(id)
+            id = id + ext
+
         # unless it already exists, create the file or image object
         # use the CMF/Plone types if appropriate
         # it will be renamed if there is an id collision with some other
@@ -717,13 +823,15 @@ class PageEditingSupport:
                                          #'Portal File','Portal Image')):
             if guess_content_type(file.filename)[0][0:5] == 'image':
                 if self.inCMF():
-                    #XXX how ?
+                    #XXX need to use cmf/plone type here, how ?
+                    #from Products.ATContentTypes import ATFile, ATImage
+                    #folder.create(ATImage) ...
                     id = folder._setObject(id, OFS.Image.Image(id,title,'')) 
                 else:
                     id = folder._setObject(id, OFS.Image.Image(id,title,''))
             else:
                 if self.inCMF():
-                    #XXX how ?
+                    #XXX as above
                     id = folder._setObject(id, OFS.Image.File(id,title,''))
                 else:
                     id = folder._setObject(id, OFS.Image.File(id,title,''))
@@ -743,11 +851,11 @@ class PageEditingSupport:
         """
         if re.search(r'(src|href)="%s"' % file_id,self.text()): return
 
-        if hasattr(self,'uploads'): folder = 'uploads/'
+        if safe_hasattr(self,'uploads'): folder = 'uploads/'
         else: folder = ''
 
         if content_type[0:5] == 'image' and \
-           not (hasattr(REQUEST,'dontinline') and REQUEST.dontinline) and \
+           not (safe_hasattr(REQUEST,'dontinline') and REQUEST.dontinline) and \
            size <= LARGE_FILE_SIZE :
             linktxt = self.pageType().inlineImage(self, file_id, folder+file_id)
         else:
@@ -762,12 +870,6 @@ class PageEditingSupport:
         # To help control executable content, make sure the new page
         # acquires it's owner from the parent folder.
         self._deleteOwnershipAfterAdd()
-        #if not self.usingRegulations():
-        #    # To help control executable content, make sure the new page
-        #    # acquires it's owner from the parent folder.
-        #    self._deleteOwnershipAfterAdd()
-        #else:
-        #    self._setOwnerRole(REQUEST)
             
     # for IssueNo0157
     _old_read = DTMLDocument.read
@@ -833,7 +935,7 @@ class PageEditingSupport:
         prop = 'max_anonymous_links'
         # we'll handle either an int or string property
         if (not self.requestHasUsername(REQUEST) and
-            hasattr(self.folder(), prop)):
+            safe_hasattr(self.folder(), prop)):
             try: max = int(getattr(self.folder(), prop))
             except ValueError: max = None
             if max is not None:
@@ -845,7 +947,7 @@ class PageEditingSupport:
         # XXX simplify ? one property for both ?
         prop = 'max_identified_links'
         # we'll handle either an int or string property
-        if (hasattr(self.folder(), prop)):
+        if (safe_hasattr(self.folder(), prop)):
             try: max = int(getattr(self.folder(), prop))
             except ValueError: max = None
             if max is not None:
@@ -855,8 +957,11 @@ class PageEditingSupport:
 
     def cleanupText(self, t):
         """
-        Do some cleanup of incoming text, also block spam links.
+        Clean up incoming text and convert it to unicode.
         """
+        # we expect utf8-encoded text always, refuse anything else early
+        unicode(t,'utf-8')
+
         # strip any browser-appended ^M's
         t = re.sub('\r\n', '\n', t)
 
@@ -884,17 +989,19 @@ class PageEditingSupport:
         if DISABLE_JAVASCRIPT:
             t = re.sub(javascriptexpr,r'&lt;disabled \1&gt;',t)
 
-        # strip out HTML header tags if present
-        def onlyBodyFrom(t):
-            # XXX these can be expensive, for now just skip if there's a problem
-            try:
-                t = re.sub(htmlheaderexpr,'',t)
-                t = re.sub(htmlfooterexpr,'',t)
-            except RuntimeError: pass
-            return t
-            # maybe better, but more inclined to mess with valid text ?
-            #return re.sub(htmlbodyexpr, r'\1', t)
-        t = onlyBodyFrom(t)
+        # for convenient pasting from external html editors: discard
+        # anything outside of HTML body tags, if they are present
+        # Actually, gets in the way when showing HTML examples; disabled.
+#         def onlyBodyFrom(t):
+#             # XXX these can be expensive, for now just skip if there's a problem
+#             try:
+#                 t = re.sub(htmlheaderexpr,'',t)
+#                 t = re.sub(htmlfooterexpr,'',t)
+#             except RuntimeError: pass
+#             return t
+#             # maybe better, but more inclined to mess with valid text ?
+#             #return re.sub(htmlbodyexpr, r'\1', t)
+#         t = onlyBodyFrom(t)
 
         return t
 
@@ -910,22 +1017,22 @@ class PageEditingSupport:
             # if we have no REQUEST, at least update last editor
             self.last_editor_ip = ''
             self.last_editor = ''
-        self.last_edit_time = DateTime(time.time()).toZone('UTC').ISO()
+        self.last_edit_time = DateTime().ISO8601()
 
     def hasCreatorInfo(self):
         """
         True if this page already has creator attributes.
         """
-        return (hasattr(self,'creator') and
-                hasattr(self,'creation_time') and
-                hasattr(self,'creator_ip'))
+        return (safe_hasattr(self,'creator') and
+                safe_hasattr(self,'creation_time') and
+                safe_hasattr(self,'creator_ip'))
                 
 
     def setCreator(self, REQUEST=None):
         """
         record my creator, creator_ip & creation_time
         """
-        self.creation_time = DateTime(time.time()).toZone('UTC').ISO()
+        self.creation_time = DateTime().ISO8601()
         if REQUEST:
             self.creator_ip = REQUEST.REMOTE_ADDR
             self.creator = self.usernameFrom(REQUEST)
@@ -968,8 +1075,8 @@ class PageEditingSupport:
         username = self.usernameFrom(REQUEST)
         if (timeStamp is not None and
             timeStamp != self.timeStamp() and
-            (not hasattr(self,'last_editor') or
-             not hasattr(self,'last_editor_ip') or
+            (not safe_hasattr(self,'last_editor') or
+             not safe_hasattr(self,'last_editor_ip') or
              username != self.last_editor or
              REQUEST.REMOTE_ADDR != self.last_editor_ip)):
             return 1
