@@ -27,7 +27,7 @@ from email.Utils import parseaddr, getaddresses
 from email.Iterators import typed_subpart_iterator
 
 from Products.ZWiki.Regexps import wikiname1,wikiname2,bracketedexpr,urlchars
-from Products.ZWiki.Utils import BLATHER
+from Products.ZWiki.Utils import BLATHER, safe_hasattr
 from Products.ZWiki.plugins.tracker.tracker import ISSUE_SEVERITIES
 DEFAULT_SEVERITY = ISSUE_SEVERITIES[len(ISSUE_SEVERITIES)/2]
 
@@ -118,8 +118,8 @@ class MailIn:
         This perhaps should do the isJunk test up front to avoid
         unnecessary resource usage.
         """
-        BLATHER('mailin.py processing incoming message:\n%s' % message)
-        #BLATHER('mailin.py processing incoming message')
+        #BLATHER('mailin.py processing incoming message:\n%s' % message)
+        BLATHER('mailin.py processing incoming message')
         self.context = context
         self.original = message
         self.msg = email.message_from_string(self.original)
@@ -151,12 +151,15 @@ class MailIn:
 
         # raises an exception if there's no text part
         try:
-            plaintextpart = typed_subpart_iterator(self.msg,
-                                                   'text',
-                                                   'plain').next().get_payload(decode=1)
+            firstplaintextpart = typed_subpart_iterator(self.msg,
+                                                        'text',
+                                                        'plain').next()
+            payload = firstplaintextpart.get_payload(decode=1)
+            content_encoding = self.msg.get_content_charset('ascii')
+            payloadutf8 = payload.decode(content_encoding).encode('utf-8')
         except StopIteration:
-            plaintextpart = ''
-        self.body = self.cleanupBody(plaintextpart)
+            payloadutf8 = ''
+        self.body = self.cleanupBody(payloadutf8)
         
     def isJunk(self):
         """
@@ -229,7 +232,7 @@ class MailIn:
         
         """
         # update the banned links property
-        if hasattr(self.folder(),'banned_links'):
+        if safe_hasattr(self.folder(),'banned_links'):
             banned_links = list(self.folder().banned_links)
             spam_links = re.findall(
                 r'((?:http|https|ftp|mailto|file):/*%s)' % urlchars,
@@ -243,7 +246,7 @@ class MailIn:
             if added_links:
                 # update property - need to find the real  owner
                 folder = self.folder()
-                while not hasattr(folder.aq_base,'banned_links'):
+                while not safe_hasattr(folder.aq_base,'banned_links'):
                     folder = folder.aq_parent
                 folder.manage_changeProperties(banned_links=banned_links)
                 # log, also try to notify admin by mail, for now
