@@ -1,13 +1,12 @@
 # PageMailSupport mixin
 
-import string, re, sys
-from string import split,join,find,lower,rfind,atoi,strip,lstrip
+import re, sys
 from types import *
 
 from I18n import _
 from TextFormatter import TextFormatter
 from Utils import html_unquote,BLATHER,formattedTraceback,stripList, \
-     isIpAddress,isEmailAddress,isUsername
+     isIpAddress,isEmailAddress,isUsername,safe_hasattr
 from Defaults import AUTO_UPGRADE, PAGE_METATYPE
 
 
@@ -40,7 +39,7 @@ class PageSubscriptionSupport:
         """
         if AUTO_UPGRADE: self._upgradeSubscribers()
         if parent:
-            if hasattr(self.folder(),'subscriber_list'):
+            if safe_hasattr(self.folder(),'subscriber_list'):
                 return stripList(self.folder().subscriber_list)
             else:
                 return []
@@ -80,16 +79,16 @@ class PageSubscriptionSupport:
 
         # migrate an old zwiki subscribers or wikifornow _subscribers attribute
         oldsubs = None
-        if (hasattr(f, 'subscribers') and
+        if (safe_hasattr(f, 'subscribers') and
             type(f.subscribers) is StringType):
             if f.subscribers:
-                oldsubs = split(re.sub(r'[ \t]+',r'',f.subscribers),',')
+                oldsubs = re.sub(r'[ \t]+',r'',f.subscribers).split(',')
             try:
                 del f.subscribers
             except KeyError:
                 BLATHER('failed to delete self.folder().subscribers')
             changed = 1
-        elif hasattr(f, '_subscribers'):
+        elif safe_hasattr(f, '_subscribers'):
             oldsubs = f._subscribers.keys()
             try:
                 del f._subscribers
@@ -97,7 +96,7 @@ class PageSubscriptionSupport:
                 BLATHER('failed to delete self.folder()._subscribers')
             changed = 1
         # ensure a subscriber_list attribute
-        if not hasattr(f, 'subscriber_list'): f.subscriber_list = []
+        if not safe_hasattr(f, 'subscriber_list'): f.subscriber_list = []
         # transfer old subscribers to subscriber_list, unless it's already
         # populated in which case discard them
         if oldsubs and not f.subscriber_list: f.subscriber_list = oldsubs
@@ -121,10 +120,10 @@ class PageSubscriptionSupport:
 
         # migrate an old zwiki subscribers attribute
         oldsubs = None
-        if (hasattr(self, 'subscribers') and
+        if (safe_hasattr(self, 'subscribers') and
             type(self.subscribers) is StringType):
             if self.subscribers:
-                oldsubs = split(re.sub(r'[ \t]+',r'',self.subscribers),',')
+                oldsubs = re.sub(r'[ \t]+',r'',self.subscribers).split(',')
             try:
                 del self.subscribers
             except KeyError:
@@ -138,7 +137,7 @@ class PageSubscriptionSupport:
 
         # migrate a wikifornow _subscribers attribute
         oldsubs = None
-        if hasattr(self, '_subscribers'):
+        if safe_hasattr(self, '_subscribers'):
             oldsubs = self._subscribers.keys()
             try:
                 del self._subscribers
@@ -246,7 +245,7 @@ class PageSubscriptionSupport:
         If parent flag is true, remove it from the parent folder's
         subscriber list instead.
         """
-        subscriber = string.lower(email)
+        subscriber = email.lower()
         if self.isSubscriber(subscriber,parent):
             sl = self._getSubscribers(parent)
             for s in sl:
@@ -396,12 +395,12 @@ class PageSubscriptionSupport:
             if not member:
                 # also check for a pseudo-member (a user acquired from above)
                 # NB doesn't work with CMFMember
-                if hasattr(memberdata,'_members'):
+                if safe_hasattr(memberdata,'_members'):
                     member = memberdata._members.get(subscriber,None)
             email = getattr(member,'email','')
         else:
             email = ''
-        return string.lower(email) or None
+        return email.lower() or None
 
     def emailAddressesFrom(self,subscribers):
         """
@@ -544,7 +543,7 @@ class PageMailSupport:
 
         Expects a list of recipient addresses.
         """
-        return join(stripList(recipients), ', ')
+        return ', '.join(stripList(recipients))
 
     def subjectHeader(self,subject='',subjectSuffix=''):
         """
@@ -572,10 +571,10 @@ class PageMailSupport:
             # page name has been suppressed
             pagename = ''
         return (
-            strip(getattr(self.folder(),'mail_subject_prefix','')) +
+            getattr(self.folder(),'mail_subject_prefix','').strip() +
             pagename +
             subject +
-            strip(subjectSuffix))
+            subjectSuffix.strip())
 
     def toHeader(self):
         """
@@ -629,9 +628,9 @@ class PageMailSupport:
         mhost = None
         folder = self.folder()
         # XXX folder might not have objectValues, don't know why (#938)
-        while (not mhost) and folder and hasattr(folder,'objectValues'):
+        while (not mhost) and folder and safe_hasattr(folder,'objectValues'):
             mhosts = folder.objectValues(
-                spec=['Mail Host', 'Secure Mail Host', 'Maildrop Host'])
+                spec=['Mail Host', 'Secure Mail Host', 'Maildrop Host', 'Secure Maildrop Host'])
             if mhosts: mhost = mhosts[0]
             folder = getattr(folder,'aq_parent',None)
         return mhost
@@ -743,7 +742,7 @@ class PageMailSupport:
         #if self.toProperty() in unique: unique.remove(self.toProperty())
 
         mailhost = self.mailhost()
-        if mailhost.meta_type == 'Secure Mail Host':
+        if mailhost.meta_type in ('Secure Mail Host', 'Secure Maildrop Host'):
             msg = text + "\n\n" +  self.signature(msgid)
             additional_headers = {
                                 'Reply-To':self.replyToHeader(), \
