@@ -44,8 +44,7 @@
 # refactor/simplify OutlineRendering
 
 from __future__ import nested_scopes
-import string, re
-from string import join
+import re
 from types import *
 from urllib import quote, unquote
 
@@ -57,7 +56,7 @@ import Persistence
 from OFS.SimpleItem import SimpleItem
 
 import Permissions
-from Utils import flatten, BLATHER
+from Utils import flatten, BLATHER, safe_hasattr
 from Defaults import PAGE_METATYPE
 from Regexps import bracketedexpr
 import Outline
@@ -212,12 +211,14 @@ class ShowSubtopicsProperty:
         if getattr(self.folder(),prop,1):
             if kw.has_key(prop):
                 return kw[prop] and 1
-            elif hasattr(self,'REQUEST') and hasattr(self.REQUEST,prop):
+            elif safe_hasattr(self,'REQUEST') and safe_hasattr(self.REQUEST,prop):
                 return getattr(self.REQUEST,prop) and 1
-            elif hasattr(self.aq_base,prop):
+            elif safe_hasattr(self.aq_base,prop):
                 return getattr(self,prop) and 1
             else:
                 primaryParent = self.primaryParent() # call only once
+                if primaryParent and primaryParent.getId() == self.getId(): # circular self-reference
+                    return 1
                 if primaryParent:
                     try: return primaryParent.subtopicsEnabled()
                     except AttributeError:
@@ -246,7 +247,7 @@ class ShowSubtopicsProperty:
         true property:   1 ("always")
         false property:  0 ("never")
         """
-        if not hasattr(self.aq_base,'show_subtopics'): return -1
+        if not safe_hasattr(self.aq_base,'show_subtopics'): return -1
         else: return self.show_subtopics and 1
 
     def setSubtopicsPropertyStatus(self,status,REQUEST=None):
@@ -302,20 +303,22 @@ class OutlineManager:
     def wikiOutline(self):
         """
         Get the outline cache which holds details of the wiki's page hierarchy.
+        """
+        self.ensureWikiOutline()
+        return self.folder().outline
+
+    def ensureWikiOutline(self):
+        """
+        Ensure this wiki has an outline cache, for fast page hierarchy.
 
         We'll generate it if needed, ie if it's missing or if it's one of
         the older types which get lost during a folder rename or when
-        moving pages to a new folder.
-
-        This is called by any method which requires the outline cache, and
-        also by upgrade() on each page view, to ensure that renaming or
-        moving an old wiki just works without requiring the user to
-        manually updateWikiOutline.
+        moving pages to a new folder. This gets called before any use of
+        the outline cache, so that it is always present and current.
         """
-        if (not hasattr(self.folder().aq_base,'outline')
+        if (not safe_hasattr(self.folder().aq_base,'outline')
             or not self.folder().outline):
             self.updateWikiOutline()
-        return self.folder().outline
 
     security.declareProtected(Permissions.View, 'updateWikiOutline')
     def updateWikiOutline(self):
@@ -340,12 +343,12 @@ class OutlineManager:
         # non-SimpleItem-based PersistentOutline object, and
         # SimpleItem-based PersistentOutline object
         # a pre-0.39 outline is just an attribute, delete (but save childmap)
-        if (hasattr(self.folder().aq_base,'outline') and
+        if (safe_hasattr(self.folder().aq_base,'outline') and
             not 'outline' in self.folder().objectIds()):
             oldchildmap = self.folder().outline.childmap()
             del self.folder().outline
         # if there's no outline object, make one
-        if not hasattr(self.folder().aq_base,'outline'):
+        if not safe_hasattr(self.folder().aq_base,'outline'):
             self.folder()._setObject('outline', PersistentOutline())
             self.folder().outline.setChildmap(oldchildmap)
         # regenerate the parentmap
@@ -930,7 +933,7 @@ class OutlineRendering:
             return got
         else:
             got.append("</ul>")
-            return join(got, "\n")
+            return "\n".join(got)
 
     security.declareProtected(Permissions.View, 'nestingAsRenderList')
     def nestingAsRenderList(self, nesting, here=None, suppress_current=0,
