@@ -239,7 +239,7 @@ class PageUtils:
         If getObject() returns None (a stale catalog entry), we return None.
         """
         for attr in PAGE_METADATA:
-            if not hasattr(brain,attr):
+            if not safe_hasattr(brain,attr):
                 # incomplete brain - make a PageBrain
                 p = brain.getObject()
                 if p: return self.metadataFor(p)
@@ -328,7 +328,7 @@ class PageUtils:
     # but way too cumbersome right now
     security.declareProtected(Permissions.View, 'homeUrl')
     def homeUrl(self):
-        return self.urlForPageOrDefault('FrontPage',self.wikiUrl())
+        return self.defaultPageUrl()
 
     security.declareProtected(Permissions.View, 'contentsUrl')
     def contentsUrl(self, scroll=1):
@@ -358,7 +358,7 @@ class PageUtils:
 
     security.declareProtected(Permissions.View, 'discussionUrl')
     def discussionUrl(self):
-        p = self.pageWithName('UserDiscussion') or self.pageWithName('GeneralDiscussion')
+        p = self.pageWithName('GeneralDiscussion')
         return (p and p.pageUrl()) or ''
 
     security.declareProtected(Permissions.View, 'indexUrl')
@@ -532,6 +532,23 @@ class PageUtils:
                 return 1
         return 0
 
+    def talsafe(self,s):
+        """
+        Sanitize a string for use with TAL's structure keyword.
+
+        Zope versions before 2.10 expect such data to be an ordinary
+        string.  Zope 2.10 expects it to be unicode, or to at least be
+        convertible to unicode using the default encoding.  We can't
+        guarantee the latter, so convert to unicode preemptively assuming
+        our standard encoding in that case.  (cf issue #1330)
+
+        This is idempotent, safe to call repeatedly.
+        """ 
+        if ZOPEVERSION < (2,10) or type(s) is UnicodeType:
+            return s
+        else:
+            return unicode(s, 'utf-8')
+        
 InitializeClass(PageUtils)
 
 
@@ -540,6 +557,7 @@ InitializeClass(PageUtils)
 #logging
 def STDERR(*args):  sys.stderr.write(' '.join(map(str,args)) + '\n')
 def BLATHER(*args): zLOG.LOG('ZWiki',zLOG.BLATHER,' '.join(map(str,args)))
+def INFO(*args):    zLOG.LOG('ZWiki',zLOG.INFO,' '.join(map(str,args)))
 def WARN(*args):    zLOG.LOG('ZWiki',zLOG.WARNING,' '.join(map(str,args)))
 def DEBUG(*args):   zLOG.LOG('ZWiki',zLOG.DEBUG,' '.join(map(str,args)))
 
@@ -548,6 +566,19 @@ def formattedTraceback():
     try:     return join(traceback.format_exception(type,val,tb),'')
     finally: del tb  # Clean up circular reference, avoid IssueNo0536
 
+def safe_hasattr(obj, name, _marker=object()): 
+    """
+    Make sure we don't mask exceptions like hasattr(). 
+    We don't want exceptions other than AttributeError to be masked, 
+    since that too often masks other programming errors. 
+    Three-argument getattr() doesn't mask those, so we use that to 
+    implement our own hasattr() replacement.
+    Boldly lifted this from Dieter Maurer, see here:
+    http://www.zope.org/Collectors/Zope/742
+    """
+    return getattr(obj, name, _marker) is not _marker
+
+
 from cgi import escape
 def html_quote(s): return escape(str(s))
 def html_unquote(s,
@@ -634,9 +665,9 @@ def flattenDtmlParse(i):
     if type(i) in (ListType,TupleType):
         if len(i) > 0: flatList.extend(flattenDtmlParse(i[0]))
         if len(i) > 1: flatList.extend(flattenDtmlParse(i[1:]))
-    elif hasattr(i,'section'):
+    elif safe_hasattr(i,'section'):
         flatList.extend(flattenDtmlParse(i.section))
-    elif hasattr(i,'im_self'):
+    elif safe_hasattr(i,'im_self'):
         flatList.extend(flattenDtmlParse(i.im_self))
     else:
         flatList.append(i)
@@ -817,3 +848,12 @@ def callHooks(hooks, arg):
                 'could not call hook, skipping (traceback follows)\n%s' % (
                 formattedTraceback()))
     return err
+
+# provide sorted for python 2.3
+if not safe_hasattr(__builtins__,'sorted'):
+    def sorted(L):
+        L = L[:]
+        L.sort()
+        return L
+else:
+    sorted = sorted
