@@ -3,29 +3,28 @@ Zwiki's skin mechanism and main UI view methods.
 
 Zwiki has a built-in skin mechanism which aims to just work and be easily
 customizable, whether in standard zope, CMF or Plone, without requiring
-any extra products or setup. Here's the scoop:
+any extra products or setup. Here are details:
 
 View methods
 ------------
 *view methods* are methods which render a particular screen in Zwiki's UI
-- editform, diffform, the main page view, etc. These define the standard
-wiki page views which are always available no matter what kind of site we
-are in. Three examples: the main page view, the edit form, the backlinks
-form, rendered by __call__(), editform(), and backlinks() respectively. We
-also have helper view methods which render smaller parts of the UI. Most
-view methods are defined by the SkinViews mixin below; others are defined
-by plugins.
+- for example the main page view, /editform, /backlinks (rendered by
+__call__(), editform(), and backlinks() respectively). These define the
+standard wiki page views which are always available no matter what kind of
+site we are in. There are also helper methods which render smaller parts
+of the UI. Most view methods are defined by the SkinViews mixin below,
+while others are defined by plugins.
 
 Skin templates
 --------------
 view methods use a *skin template* to control their rendering.  This is a
-page template, dtml method or file of the same name as the method (perhaps
-with a suffix added). It is found by looking first in the ZODB (current
-folder, acquisition context, CMF skin layers), then on the filesystem
-(plugins, ZWiki/skins/<current skin>, ZWiki/skins/zwiki) - see
-getSkinTemplate for more details. The default zwiki skin provides all
-the standard templates, and these may be overridden selectively.
-They are listed here: http://zwiki.org/QuickReference#skin-templates
+page template, dtml method or file of the same name as the method
+(optionally with the standard suffix added). It is found by looking first
+in the ZODB (current folder, acquisition context, CMF skin layers), then
+on the filesystem (plugins, ZWiki/skins/<current skin>, ZWiki/skins/zwiki)
+- see getSkinTemplate for more details. The default zwiki skin provides
+all the standard templates, and these may be overridden selectively.  They
+are listed at http://zwiki.org/QuickReference#skin-templates .
 
 Skin macros
 -----------
@@ -77,23 +76,20 @@ Other notes
   RecentChanges.dtml.
 
 - the stylesheet method looks for a skin template object named
-  ``stylesheet.css`` of ``stylesheet``, in that order.  See also
+  ``stylesheet.css`` or ``stylesheet`` in that order.  See also
   http://zwiki.org/HowToSetUpAnEditableStylesheet .
 
 - when running zope in debug mode, all filesystem-based skin templates and
-  macros will refresh, showing changes immediately without a zope restart.
+  macros will show changes immediately without a zope restart.
 
 """
 
 
 from __future__ import nested_scopes
-import os, sys, re, string, time, math
-import string
-from string import split,join,find,lower,rfind,atoi,strip
+import os
 
 from App.Common import rfc1123_date
 from AccessControl import getSecurityManager, ClassSecurityInfo
-import Permissions
 from OFS.Image import File
 from Globals import InitializeClass, MessageDialog
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
@@ -101,8 +97,9 @@ from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
 from Products.PageTemplates.Expressions import SecureModuleImporter
 from ComputedAttribute import ComputedAttribute
 
+import Permissions
 from Defaults import PAGE_METATYPE
-from Utils import BLATHER, formattedTraceback, abszwikipath
+from Utils import BLATHER, formattedTraceback, abszwikipath, safe_hasattr
 from I18n import _, DTMLFile, HTMLFile
 
 
@@ -126,7 +123,7 @@ def loadDtmlMethod(name,dir='skins/zwiki'):
         #dm = DTMLFile(os.path.join(dir,name), globals())
         dm = HTMLFile(f, globals())
         # work around some (2.7 ?) glitch
-        if not hasattr(dm,'meta_type'): dm.meta_type = 'DTML Method (File)'
+        if not safe_hasattr(dm,'meta_type'): dm.meta_type = 'DTML Method (File)'
         return dm
     else:
         return None
@@ -211,6 +208,7 @@ for s in os.listdir(abszwikipath('skins')):
         'denied',
         'diffform',
         'editform',
+        'history',
         'recentchanges',
         'searchwiki',
         'helppage',
@@ -307,6 +305,8 @@ class SkinViews:
         """
         Render the backlinks form (template-customizable).
         """
+        self.ensureCatalog()
+        self.ensureWikiOutline()
         return self.getSkinTemplate('backlinks')(self,REQUEST)
 
     security.declareProtected(Permissions.View, 'contentspage')
@@ -316,11 +316,12 @@ class SkinViews:
 
         hierarchy and singletons parameters are required.
         """
+        self.ensureWikiOutline()
         return self.getSkinTemplate('contentspage')(self,REQUEST,
                                                     hierarchy=hierarchy,
                                                     singletons=singletons)
 
-    security.declareProtected(Permissions.Add, 'createform')
+    security.declarePublic('createform')      # check permissions at runtime
     def createform(self, REQUEST=None, page=None, text=None, pagename=None):
         """
         Render the create form (template-customizable).
@@ -331,6 +332,9 @@ class SkinViews:
         It may also be customized by a createform skin template, in
         which case page creation and page editing forms are different.
         """
+        if not self.checkPermission(Permissions.Add, self.folder()):
+            raise 'Unauthorized', (
+                _('You are not authorized to add pages in this wiki.'))
         if not self.checkSufficientId(REQUEST):
             return self.denied(
                 _("Sorry, this wiki doesn't allow anonymous edits. Please configure a username in options first."))
@@ -377,15 +381,23 @@ class SkinViews:
         return self.getSkinTemplate('denied')(self,REQUEST,reason=reason)
 
     security.declareProtected(Permissions.View, 'diffform')
-    def diffform(self, revA, difftext, REQUEST=None):
+    def diffform(self, rev, difftext, bodytext, REQUEST=None):
         """
         Render the diff form (template-customizable).
 
-        revA and difftext parameters are required.
+        rev and difftext parameters are required.
         """
         return self.getSkinTemplate('diffform')(self,REQUEST,
-                                                 revA=revA,
-                                                 difftext=difftext)
+                                                rev=rev,
+                                                difftext=difftext,
+                                                bodytext=bodytext)
+
+    security.declareProtected(Permissions.View, 'history')
+    def history(self, REQUEST=None):
+        """
+        Render the edit history view (template-customizable).
+        """
+        return self.getSkinTemplate('history')(self,REQUEST)
 
     security.declareProtected(Permissions.View, 'editConflictDialog')
     def editConflictDialog(self):
@@ -439,7 +451,7 @@ class SkinViews:
                 _("Sorry, this wiki doesn't allow anonymous edits. Please configure a username in options first."))
         
         if ((not page or page == self.pageName()) and
-            hasattr(self,'wl_isLocked') and self.wl_isLocked()):
+            safe_hasattr(self,'wl_isLocked') and self.wl_isLocked()):
             return self.davLockDialog()
 
         # what are we going to do ? set up page, text & action accordingly
@@ -474,6 +486,7 @@ class SkinViews:
         """
         Render the recentchanges form (template-customizable).
         """
+        self.ensureCatalog()
         return self.getSkinTemplate('recentchanges')(self,REQUEST)
 
     # we call this searchwiki, not searchpage, for clarity
@@ -482,6 +495,7 @@ class SkinViews:
         """
         Render the searchwiki form (template-customizable).
         """
+        self.ensureCatalog()
         return self.getSkinTemplate('searchwiki')(self,REQUEST)
 
     searchpage = searchwiki # alias
@@ -691,7 +705,7 @@ class SkinUtils:
         applied or not.
         """
         REQUEST = getattr(self,'REQUEST',None)
-        if (hasattr(REQUEST,'bare') or kw.has_key('bare')):
+        if (safe_hasattr(REQUEST,'bare') or kw.has_key('bare')):
             return body
         else:
             return self.getSkinTemplate('wikipage')(self,REQUEST,body=body,**kw)
@@ -777,7 +791,7 @@ class SkinSwitchingUtils:
         if not hasSkin(skin): return
         # is the user logged in ? if not, return harmlessly
         member = portal_membership.getAuthenticatedMember()
-        if not hasattr(member,'setProperties'): return
+        if not safe_hasattr(member,'setProperties'): return
         # change their skin preference and reload page
         REQUEST.form['portal_skin'] = skin
         member.setProperties(REQUEST)
