@@ -64,17 +64,18 @@ import Permissions
 from Defaults import AUTO_UPGRADE, IDS_TO_AVOID, \
      PAGE_METATYPE, LINK_TO_ALL_CATALOGED, LINK_TO_ALL_OBJECTS, \
      WIKINAME_LINKS, BRACKET_LINKS, DOUBLE_BRACKET_LINKS, \
-     DOUBLE_PARENTHESIS_LINKS, ISSUE_LINKS, \
+     DOUBLE_PARENTHESIS_LINKS, ISSUE_LINKS, PAGE_METADATA, \
      CONDITIONAL_HTTP_GET, CONDITIONAL_HTTP_GET_IGNORE
 from Regexps import url, bracketedexpr, singlebracketedexpr, \
      doublebracketedexpr, doubleparenthesisexpr, wikiname, wikilink, \
      interwikilink, remotewikiurl, protected_line, zwikiidcharsexpr, \
      anywikilinkexpr, markedwikilinkexpr, localwikilink, \
      spaceandlowerexpr, dtmlorsgmlexpr, wikinamewords, hashnumberexpr
-from Utils import PageUtils, BLATHER, DateTimeSyntaxError
+from Utils import PageUtils, BLATHER, DateTimeSyntaxError, safe_hasattr
 from Views import PageViews
 from OutlineSupport import PageOutlineSupport
-from Diff import PageDiffSupport
+from Diff import PageDiffSupport # XXX to be replaced by..
+from History import PageHistorySupport
 from Mail import PageSubscriptionSupport, PageMailSupport
 from Catalog import PageCatalogSupport
 from CMF import PageCMFSupport
@@ -90,9 +91,12 @@ from plugins import PLUGINS
 
 DEFAULT_PAGETYPE = PAGETYPES[0]
 
-
+# see plugins/__init__.py    
+#
+# PageCMFSupport is last to avoid PortalContent.id overriding
+# DTMLDocument.id() as older code expects ZWikiPage.id() to be callable.
 class ZWikiPage(    
-    PLUGINS[0], # see plugins/__init__.py    
+    PLUGINS[0],
     PLUGINS[1],
     PLUGINS[2],
     PLUGINS[3],
@@ -111,6 +115,7 @@ class ZWikiPage(
     PageEditingSupport, 
     PageOutlineSupport,
     PageDiffSupport,
+    PageHistorySupport,
     PageMailSupport,
     PageSubscriptionSupport,
     PageCatalogSupport,
@@ -120,8 +125,6 @@ class ZWikiPage(
     PageUtils,
     PageViews,
     DTMLDocument,
-    # XXX last to avoid PortalContent.id overriding DTMLDocument.id(),
-    # as older code expects ZWikiPage.id() to be callable.
     PageCMFSupport,
     ):
     """
@@ -164,6 +167,7 @@ class ZWikiPage(
         {'id':'last_editor_ip', 'type': 'string', 'mode': 'r'},
         {'id':'last_edit_time', 'type': 'string', 'mode': 'r'},
         {'id':'last_log', 'type': 'string', 'mode': 'r'},
+        {'id':'revision_number', 'type': 'int', 'mode': 'r'},
         ) \
         + PageOutlineSupport._properties \
         + PageSubscriptionSupport._properties \
@@ -178,6 +182,7 @@ class ZWikiPage(
     last_editor_ip = ''
     last_edit_time = ''
     last_log = ''
+    revision_number = 1
     PAGE_TYPES = PAGE_TYPES # used by skin templates
     # page_type used to be a string used to select a render method. 
     # As of 0.25 it is an object which encapsulates the page's formatting
@@ -197,7 +202,7 @@ class ZWikiPage(
             BLATHER("upgrading %s's page type from %s to %s" % (self.id(),self.page_type,new))
             self.setPageType(new)
         # page type whose plugin is no longer installed ?
-        elif not hasattr(self.page_type,'render'):
+        elif not safe_hasattr(self.page_type,'render'):
             self.setPageType(DEFAULT_PAGETYPE)
         return self.page_type
     def lookupPageType(self,id=None):
@@ -265,7 +270,7 @@ class ZWikiPage(
         if not self.preRendered(): self.preRender()
         r = self.pageType().render(self, REQUEST, RESPONSE, **kw)
         if RESPONSE:
-            if hasattr(self,'zwiki_content_type'):
+            if safe_hasattr(self,'zwiki_content_type'):
                 RESPONSE.setHeader('content-type',getattr(self,'zwiki_content_type'))
             elif not RESPONSE.getHeader('content-type'):
                 RESPONSE.setHeader('content-type','text/html')
@@ -293,7 +298,7 @@ class ZWikiPage(
         forcibly clear out any cached render data for this page
         """
         self.setPreRendered('')
-        if hasattr(self,'_v_cooked'):
+        if safe_hasattr(self,'_v_cooked'):
             delattr(self,'_v_cooked')
             delattr(self,'_v_blocks')
         if REQUEST: REQUEST.RESPONSE.redirect(self.pageUrl())
@@ -416,7 +421,7 @@ class ZWikiPage(
         """Is embedded DTML permitted on this page ?"""
         return (
             getattr(self,'allow_dtml',0) and
-            not hasattr(self,'no_dtml')
+            not safe_hasattr(self,'no_dtml')
             )
 
     security.declareProtected(Permissions.View, 'supportsEpoz')
@@ -427,7 +432,7 @@ class ZWikiPage(
     security.declareProtected(Permissions.View, 'epozInstalled')
     def epozInstalled(self):
         """Is Epoz installed ?"""
-        return hasattr(self,'Epoz')
+        return safe_hasattr(self,'Epoz')
 
     def handle_modified_headers(self, last_mod=None, REQUEST=None):
         """
@@ -629,6 +634,7 @@ class ZWikiPage(
                 withinSgmlOrDtml(match.span(),text)):
                 return link
         link = linkorig = re.sub(markedwikilinkexpr, r'\1', link)
+        label = None
 
         # here we go
         # is this link escaped ?
@@ -655,7 +661,8 @@ class ZWikiPage(
 
         # is it a bare URL ?
         if re.match(url,link):
-            return '<a href="%s">%s</a>' % (link, link)
+            label = re.sub(r'^mailto:','',link)
+            return '<a href="%s">%s</a>' % (link, label)
 
         # is it a hash number issue link (#123) ?
         if re.match(hashnumberexpr,link):
@@ -678,26 +685,41 @@ class ZWikiPage(
             # no - we have exhausted our linking arsenal, give up
             return linkorig
 
-        # is it a freeform wiki link ?
+        # is it a freeform (bracketed) wiki link ?
         if not self.isWikiName(link):
             # yes - use fuzzy matching to match an existing page if possible.
             # strip brackets/parentheses
             link = stripDelimitersFrom(linkorig)
+            # XXX experimental MW-style link labels
+            def linkTargetAndLabel(link):
+                """Return target and label from link text possibly containing |."""
+                l = link.split("|")
+                if len(l) < 2: l = l + ['']
+                if len(l) > 2: l = [l[0], '|'.join(l[1:])]
+                return l[0], l[1]
+            link, label = linkTargetAndLabel(link)
+            # end
             p = self.pageWithFuzzyName(link)
             if p:
+                # found the page, use its id for linking
                 try: link = p.getId() # XXX poor caching
                 except AttributeError: link = p.id   # all-brains
-                # and fall through
-            
-        # must be either a WikiName link, or an existing page's id from above
+            else:
+                # no such page, maybe this is an external link ?
+                if re.match(url,link):
+                    label = re.sub(r'^mailto:','',label)
+                    return '<a href="%s">%s</a>' % (link, label)
+                # no - treat it as an uncreated page
+
         return self.renderLinkToPage(link,
                                      linkorig=linkorig,
                                      link_title=link_title,
-                                     access_key=access_key)
+                                     access_key=access_key,
+                                     label=label)
 
     # XXX helper for above
     def renderLinkToPage(self,page,linkorig=None,link_title=None,
-                         access_key=None,name=None):
+                         access_key=None,name=None,label=None):
                          
         """
         Render a wiki link to page, which may or may not exist.
@@ -725,6 +747,7 @@ class ZWikiPage(
             else:
                 style = ''
             link      = stripDelimitersFrom(linkorig or page)
+            label     = label and label or self.formatWikiname(link)
             return '<a href="%s/%s"%s%s%s%s>%s</a>' % (
                 self.wikiUrl(),
                 quote(page),
@@ -732,7 +755,7 @@ class ZWikiPage(
                 name,
                 accesskey,
                 style,
-                self.formatWikiname(link))
+                label)
         else:
             # no - provide a creation link
             return (
@@ -877,7 +900,7 @@ class ZWikiPage(
 
             # build the link around the interval
             linked_interval = (
-                ' <a href="%s/diff" title="%s%s" >%s</a>' % (
+                ' <a href="%s/history" title="%s%s" >%s</a>' % (
                 self.pageUrl(),
                 _('show last edit'),
                 lastlog, 
@@ -1021,6 +1044,11 @@ class ZWikiPage(
         """
         return self.getPath()[:self.getPath().rfind('/')]
 
+    def hasAllCatalogFields(self):
+        """Does our catalog have all the fields needed for fast queries ?"""
+        return self.hasCatalogIndexesMetadata(
+            (['meta_type','path'],PAGE_METADATA))
+
     security.declareProtected(Permissions.View, 'pages')
     def pages(self, **kw):
         """
@@ -1054,23 +1082,15 @@ class ZWikiPage(
         insensitive, partial matching in page names and page text.
         
         """
-        if self.hasCatalogIndexesMetadata((['meta_type','path'], [])):
-            if self.linkToAllCataloged():
-                # look at all cataloged pages ?
-                return filter(lambda x:x is not None,
-                              map(lambda x:self.ensureCompleteMetadataIn(x),
-                                  self.searchCatalog(meta_type=self.meta_type,
-                                                     **kw)))
-            else:
-                # or (usually) just the ones in this folder
-                wikipath = self.wikiPath()
-                def folderpath(s): return s[:s.rfind('/')]
-                return filter(lambda x:x is not None,
-                              map(lambda x:self.ensureCompleteMetadataIn(x),
-                                  filter(lambda x:folderpath(x.getPath())==wikipath,
-                                         self.searchCatalog(meta_type=self.meta_type,
-                                                            path=wikipath,
-                                                            **kw))))
+        if self.hasAllCatalogFields():
+            wikipath = self.wikiPath()
+            def inthiswiki(b):
+                p = b.getPath()
+                return p[:p.rfind('/')] == wikipath
+            return [b for b in
+                    self.searchCatalog(meta_type=self.meta_type,path=wikipath,**kw)
+                    if inthiswiki(b)]
+
         else:
             results = []
             for p in self.pageObjects(): results.append(self.metadataFor(p))
@@ -1120,10 +1140,8 @@ class ZWikiPage(
         If there's junk in the catalog, pages could return a page with id
         None; we guard against that.
         """
-        # faster, & more robust
-        return self.folder().objectIds(spec=self.meta_type) 
-        # catalog-driven, so could work across folders (cf ALLBRAINS etc.)
-        #return filter(lambda x:x is not None, map(lambda x:x.id,self.pages())) 
+        # note btreefolders don't always give a list
+        return list(self.folder().objectIds(spec=self.meta_type))
 
     security.declareProtected(Permissions.View, 'pageNames')
     def pageNames(self):
@@ -1282,36 +1300,41 @@ class ZWikiPage(
         """
         Return the page in this folder for which name is a fuzzy link, or None.
 
-        A fuzzy link ignores whitespace, case and punctuation.  If there
-        are multiple fuzzy matches, return the page whose name is
-        alphabetically first.  The allow_partial flag allows even fuzzier
-        matching. As of 0.17 ignore_case is not used and kept only for
-        backward compatibility. numeric_match modifies partial matching so
-        that [1] does not match a page "12...".
+        A fuzzy link ignores capitalization, punctuation and whitespace.
+        If there are multiple fuzzy matches, return the page whose name is
+        alphabetically first.
 
-        performance-sensitive
+        The allow_partial flag allows even fuzzier matching. numeric_match
+        modifies partial matching so that numbers match exactly, eg 1 does
+        not match a page named 100.  ignore_case is not used and kept only
+        for backward compatibility.
+
+        performance-sensitive, at least the allow_partial=0 case
         """
-        if url_quoted:
-            name = unquote(name)
+        if url_quoted: name = unquote(name)
+
+        # try common fast case first
         p = self.pageWithName(name)
         if p: return p
+
+        # not quite as simple as you'd think
         id = self.canonicalIdFrom(name)
-        idlower = string.lower(id)
-        ids = self.pageIds()
-        # in case this is a BTreeFolder (& old zope ?), work around as per
-        # IssueNo0535PageWithFuzzyNameAndBTreeFolder.. may not return
-        # the alphabetically first page in this case
-        try:
-            ids.sort()
-        except AttributeError:
-            pass
+        idlower = id.lower()
+        if allow_partial and name.isdigit():
+            names = self.pageNames()
+            names.sort()
+            ids = [self.canonicalIdFrom(name) for name in names]
+        else:
+            ids = self.pageIds()
+
         for i in ids:
-            ilower = string.lower(i)
+            ilower = i.lower()
             if (ilower == idlower or 
                 ((allow_partial and ilower[:len(idlower)] == idlower) and not
                  (numeric_match and re.match(r'[0-9]',ilower[len(idlower):])))
                 ):
                 return self.pageWithId(i)
+
         return None
         
     security.declareProtected(Permissions.View, 'backlinksFor')
@@ -1462,12 +1485,26 @@ def within_literal(upto, after, state, text,
 
     # Check whether '<a href...>' is currently (possibly, still) prevailing.
     #XXX make this more robust
-    opening = rfind(text, '<a href', lastend, upto)
-    if (opening != -1) or inanchor:
+    opening = rfind(text, '<a ', lastend, upto)
+    got_anchor = opening != -1
+    # we used to be looking for '<a href', but attribute order can
+    # actually be the other way around, e.g. RST generating <a name=...
+    href_too = False
+    if got_anchor or inanchor: # found or already in anchor
+        if got_anchor: # found it here
+            # it might just have been <a name...>
+            href_too = -1 != rfind(text, 'href', opening, upto)
+            # if not href_too: opening = -1
         if opening != -1: opening = opening + 5
         else: opening = lastend
-        if -1 == rfind(text, '</a>', opening, upto):
-            newinanchor = 1
+        got_anchor_closing = -1 == rfind(text, '</a>', opening, upto)
+        if href_too and got_anchor_closing:
+            newinanchor = 1 # the <a name=... href=...>WikiName</a> case
+        elif got_anchor_closing:
+            newinanchor = 0 # the <a name>WikiName</a> case
+        # this would be: elif got_anchor and not got_anchor_closing:
+            # the "<a name>WikiName - no closing" case is handled implicitly
+            # newinanchor stays the same as initialized = 0
     state['inanchor'] = newinanchor
 
     state['lastend'] = after
