@@ -9,6 +9,13 @@ def test_suite():
 
 class Tests(ZwikiTestCase):
 
+    def test_manage_addZWikiPage(self):
+        self.folder.manage_addProduct['ZWiki'].manage_addZWikiPage(\
+            'ZmiTestPage')
+        self.assert_('ZmiTestPage' in self.folder.objectIds('ZWiki Page'))
+        # the wiki outline object is also updated
+        self.assert_(self.folder.ZmiTestPage.wikiOutline().hasNode('ZmiTestPage'))
+
     def test_excerptAt(self):
         self.page.edit(text='This is a test of the<br />\n excerptAt method,')
         self.assertEquals(self.page.excerptAt('excerptat',size=10,highlight=0),
@@ -37,7 +44,7 @@ class Tests(ZwikiTestCase):
         self.page.setupCatalog()
         self.page.catalog().manage_delColumn('parents')
         brain = self.page.pages()[0]
-        self.assert_(hasattr(brain,'parents'))
+        self.assert_(safe_hasattr(brain,'parents'))
 
     def test_canonicalIdFrom(self):
         p = self.page
@@ -81,25 +88,25 @@ class Tests(ZwikiTestCase):
     def test_pageWithId(self):
         p = self.page
         self.failIf(p.pageWithId('nosuchid'))
-        self.failUnless(p.pageWithId('TestPage'))
+        self.assert_(p.pageWithId('TestPage'))
         self.failIf(p.pageWithId('testpage'))
-        self.failUnless(p.pageWithId('testpage',ignore_case=1))
+        self.assert_(p.pageWithId('testpage',ignore_case=1))
 
     def test_pageWithName(self):
         p = self.page
         p.title = 'Test page'
-        self.failUnless(p.pageWithName(p.title))
+        self.assert_(p.pageWithName(p.title))
 
     def test_pageWithFuzzyName(self):
         p = self.page
         p.title = 'Test page'
-        self.failUnless(p.pageWithFuzzyName('Test page'))
-        self.failUnless(p.pageWithFuzzyName(' Test  page\t'))
-        self.failUnless(p.pageWithFuzzyName('TestPage'))
-        self.failUnless(p.pageWithFuzzyName('TEST Page'))
-        self.failUnless(p.pageWithFuzzyName('Testpage'))
+        self.assert_(p.pageWithFuzzyName('Test page'))
+        self.assert_(p.pageWithFuzzyName(' Test  page\t'))
+        self.assert_(p.pageWithFuzzyName('TestPage'))
+        self.assert_(p.pageWithFuzzyName('TEST Page'))
+        self.assert_(p.pageWithFuzzyName('Testpage'))
         self.failIf(p.pageWithFuzzyName('test'))
-        self.failUnless(p.pageWithFuzzyName('test',allow_partial=1))
+        self.assert_(p.pageWithFuzzyName('test',allow_partial=1))
 
     def test_backlinksFor(self):
         p = self.page
@@ -162,16 +169,21 @@ class Tests(ZwikiTestCase):
         self.assertEquals(
             self.p.renderLink('http://some.url'),
             '<a href="http://some.url">http://some.url</a>')
-        #self.p.folder().space_wikinames = 1
-        #self.assertEquals(self.p.renderLink('CamelCase')[:10],'Camel Case')
+        self.p.folder().space_wikinames = 1
+        self.assertEquals(self.p.renderLink('CamelCase')[:10],'Camel Case')
+        self.assertEquals(
+            self.p.renderLink('[testpage | the label]'),
+            '<a href="http://nohost/test_folder_1_/wiki/TestPage"> the label</a>')
+        self.assertEquals(
+            self.p.renderLink('[http://some.url|label]'),
+            '<a href="http://some.url">label</a>')
         
     def test_renderLinksIn(self):
         self.assertEquals(self.p.renderLinksIn('nolink'),'nolink')
         self.assertEquals(self.p.renderLinksIn('http://a.b.c/d'),
                           '<a href="http://a.b.c/d">http://a.b.c/d</a>')
-        self.assertEquals(self.p.renderLinksIn('mailto://a@b.c'),
-                          '<a href="mailto://a@b.c">mailto://a@b.c</a>')
-        #import pdb; pdb.set_trace()
+        self.assertEquals(self.p.renderLinksIn('mailto:a@b.c'),
+                          '<a href="mailto:a@b.c">a@b.c</a>')
         self.assertEquals(self.p.renderLinksIn('TestPage'),
                           '<a href="http://nohost/test_folder_1_/wiki/TestPage">TestPage</a>')
 #                          '<a href="/test_folder_1_/wiki/TestPage" title="" style="background-color:;">TestPage</a>')
@@ -220,23 +232,23 @@ class Tests(ZwikiTestCase):
         self.failIf(self.p.displaysSubtopicsWithDtml())
         self.p.allow_dtml = 1
         self.p.edit(text='<dtml-var subtopics')
-        self.failUnless(self.p.displaysSubtopicsWithDtml())
+        self.assert_(self.p.displaysSubtopicsWithDtml())
         self.p.edit(text='&dtml-subtopics')
-        self.failUnless(self.p.displaysSubtopicsWithDtml())
+        self.assert_(self.p.displaysSubtopicsWithDtml())
 
     def test_linkTitleFrom(self):
         edittime = DateTime.DateTime() - 0.2
-        edittime = edittime.ISO()
+        edittime = edittime.ISO8601()
         r = self.p.linkTitleFrom()
         self.assertEquals( r, 'last edited some time ago')
         r = self.p.linkTitleFrom(prettyprint=1) 
-        self.failUnless( 'some time' in r )
-        self.failUnless( '<a href=' in r )
+        self.assert_( 'some time' in r )
+        self.assert_( '<a href=' in r )
         r = self.p.linkTitleFrom(last_edit_time=edittime, prettyprint=1)
-        self.failUnless( '4 hours' in r )
+        self.assert_( '4 hours' in r )
         r = self.p.linkTitleFrom(last_edit_time=edittime, \
                                  last_editor='fred', prettyprint=1)
-        self.failUnless( 'fred' in r )
+        self.assert_( 'fred' in r )
         edittime = 'not valid'
         r = self.p.linkTitleFrom(last_edit_time=edittime)
-        self.failUnless( 'some time' in r )
+        self.assert_( 'some time' in r )
