@@ -3,11 +3,12 @@
 __doc__="""
 ZWiki product
 """
-__version__='0.59.0'
+__version__='0.60.0'
 
-import os, re, os.path, string, urllib
+import re, os.path, string, urllib
 
 from AccessControl import getSecurityManager
+from DateTime import DateTime
 from Globals import package_home, MessageDialog, ImageFile
 from OFS.Folder import Folder
 from OFS.Image import Image, File
@@ -15,10 +16,12 @@ from OFS.ObjectManager import customImporters
 from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
 from Products.PythonScripts.PythonScript import PythonScript
 
-import Admin, Defaults, OutlineSupport, Permissions, ZWikiPage
+import Defaults, OutlineSupport, Permissions, ZWikiPage
 from Admin import addDTMLMethod
 from I18n import _, DTMLFile
 from pagetypes import PAGETYPES
+from Utils import parseHeadersBody, safe_hasattr, INFO, BLATHER, \
+     formattedTraceback
 
 
 misc_ = {
@@ -30,18 +33,6 @@ misc_ = {
     'blank_star_icon': ImageFile(os.path.join('skins','zwiki','blank_star.png'),globals()),
     }
 
-def dummyOutlineConstructor(self):
-    """Stub so we can get PersistentOutline registered with ZMI.
-
-    The real constructor is a zwikipage method and is called automatically.
-    """
-    return MessageDialog(
-        title=_("No need to add a Zwiki Outline Cache"),
-        message=_("""Zwiki Outline Cache appears in the ZMI Add menu
-        for implementation reasons, but should not be added directly.
-        Zwiki will create it for you as needed.
-        """))
-
 def initialize(context): 
     """Initialize the ZWiki product.
     """
@@ -58,14 +49,18 @@ def initialize(context):
                 ZWikiPage.manage_addZWikiPage,
                 ),
             )
-        # and the PersistentOutline class, so it's zmi-manageable
+        # also the PersistentOutline class, so it's zmi-manageable
+        def outlineConstructorStub(self):
+            return MessageDialog(
+                title=_("No need to add a Zwiki Outline Cache"),
+                message=_("""Zwiki Outline Cache appears in the ZMI Add menu
+                for implementation reasons, but should not be added directly.
+                Zwiki will create it for you as needed."""))
         context.registerClass(
             OutlineSupport.PersistentOutline,
             permission=Permissions.manage_properties,
             #icon = 'images/ZWikiPage_icon.gif',
-            constructors = (
-                dummyOutlineConstructor,
-                ),
+            constructors = (outlineConstructorStub,)
             )
         # set up an "add wiki" menu item
         context.registerClass(
@@ -83,27 +78,104 @@ def initialize(context):
                 addWikiFromZodb,
                 )
             )
-        # do CMF initialisation if installed
-        try:
-            import CMFInit
-            CMFInit.initialize(context)
-        except ImportError:
-            pass
-        # register skin as customizable dir if FileSystemSite is installed
-        try:
-            from Products.FileSystemSite.DirectoryView import registerDirectory
-            registerDirectory('skins/zwiki', globals())
-        except ImportError:
-            pass
+        initializeForCMF(context)
+        initializeForFSS(context)
 
-    # don't let any error prevent initialisation
+    # don't let any error prevent initialization
     except:
-        # log it
-        import sys, traceback, string
-        type, val, tb = sys.exc_info()
-        sys.stderr.write(string.join(traceback.format_exception(type, val, tb),
-                                     ''))
-        del type, val, tb
+        INFO('failed to initialise',formattedTraceback())
+
+# default_perms = {
+#     'create': 'nonanon',
+#     'edit': 'nonanon',
+#     'comment': 'nonanon',
+#     'move': 'owners', # rename/delete/reparent
+#     'regulate': 'owners'
+#     }
+
+def initializeForCMF(context):
+    """Do global CMF/Plone-specific initialization, if they are installed."""
+    try:
+        import Products.CMFCore.DirectoryView
+        from Products.CMFCore.utils import ContentInit
+        #from Products.CMFCore.permissions import AddPortalContent
+        #from Products.Archetypes.public import listTypes, process_types
+        from Products.CMFPlone.interfaces import IPloneSiteRoot
+        from Products.GenericSetup import EXTENSION, profile_registry
+
+        # register our GenericSetup profiles
+        profile_registry.registerProfile('default',
+                                         'ZWiki',
+                                         'Extension profile for default Zwiki-in-Plone/CMF setup',
+                                         'profiles/default',
+                                         'ZWiki',
+                                         EXTENSION,
+                                         for_=IPloneSiteRoot)
+        profile_registry.registerProfile('uninstall',
+                                         'ZWiki',
+                                         'Extension profile for removing default Zwiki-in-Plone/CMF setup',
+                                         'profiles/uninstall',
+                                         'ZWiki',
+                                         EXTENSION)
+
+        # register our skin layer(s)
+        Products.CMFCore.DirectoryView.registerDirectory('skins', globals())
+
+        # initialize portal content
+        PROJECT = 'Zwiki'
+        #types, cons, ftis = process_types(listTypes(PROJECT),PROJECT)
+        ContentInit(
+            PROJECT + ' Content',
+            content_types      = (ZWikiPage.ZWikiPage,), # types,
+            #permission         = AddPortalContent,   # Add portal content
+            permission         = Permissions.Add,     # Zwiki: Add pages
+            extra_constructors = (addWikiPageInCMF,), # cons
+            #fti                = ftis,               # ignored
+            ).initialize(context)
+
+    except ImportError:
+        INFO('failed to initialise for Plone/CMF, Plone/CMF sites will not recognise Zwiki')
+        BLATHER(formattedTraceback())
+
+# a (old-style) CMF factory method
+def addWikiPageInCMF(self, id, title='', page_type=None, file=''):
+    def makeWikiPage(id, title, file):
+        def initPageMetadata(page):
+            page.creation_date = DateTime()
+            page._editMetadata(title='',
+                               subject=(),
+                               description='',
+                               contributors=(),
+                               effective_date=None,
+                               expiration_date=None,
+                               format='text_html',
+                               language='',
+                               rights = '')
+
+        ob = ZWikiPage.ZWikiPage(source_string=file, __name__=id)
+        ob.title = title
+        ob.parents = []
+        username = getSecurityManager().getUser().getUserName()
+        ob.manage_addLocalRoles(username, ['Owner'])
+        initPageMetadata(ob)
+        return ob
+
+    id=str(id)
+    title=str(title)
+    ob = makeWikiPage(id, title, file)
+    ob.setPageType(
+        page_type or getattr(self,'allowed_page_types',[None])[0])
+    self._setObject(id, ob)
+
+def initializeForFSS(context):
+    """Do FileSystemSite-specific initialization, if it is installed."""
+    try:
+        from Products.FileSystemSite.DirectoryView import registerDirectory
+        # register our skin layer as a customizable directory
+        registerDirectory('skins/zwiki', globals())
+        INFO('registered zwiki skin layer with FileSystemSite')
+    except ImportError:
+        pass
 
 
 # set up hooks for ZMI operations on zwiki objects, for:
@@ -116,6 +188,8 @@ def initialize(context):
 # Note: manage_renameObject will lose parentage in the wiki outline (?)
 #
 # 3. catalog awareness
+#
+# XXX convert to events
 
 def manage_afterAdd(self, item, container):
     if not self.hasCreatorInfo():
@@ -175,20 +249,19 @@ ZWikiPage.ZWikiPage.manage_editProperties = manage_editProperties
 
 manage_addWikiForm = DTMLFile('skins/zwiki/addwikiform', globals())
 
-def manage_addWiki(self, new_id, new_title='', wiki_type='zwikidotorg',
+def manage_addWiki(self, new_id, new_title='', wiki_type='basic',
                        REQUEST=None, enter=0):
     """
     Create a new zwiki web of the specified type
     """
-    if not REQUEST: REQUEST = self.REQUEST
-    
     # check for a configuration wizard
-    if hasattr(self,wiki_type+'_config'):
-        REQUEST.RESPONSE.redirect('%s/%s_config?new_id=%s&new_title=%s' % \
-                                  (REQUEST['URL1'],
-                                   wiki_type,
-                                   urllib.quote(new_id),
-                                   urllib.quote(new_title)))
+    if safe_hasattr(self,wiki_type+'_config'):
+        if REQUEST:
+            REQUEST.RESPONSE.redirect('%s/%s_config?new_id=%s&new_title=%s' % \
+                                      (REQUEST['URL1'],
+                                       wiki_type,
+                                       urllib.quote(new_id),
+                                       urllib.quote(new_title)))
         
     else:
         if wiki_type in self.listFsWikis():
@@ -196,15 +269,19 @@ def manage_addWiki(self, new_id, new_title='', wiki_type='zwikidotorg',
         elif wiki_type in self.listZodbWikis():
             self.addWikiFromZodb(new_id,new_title,wiki_type,REQUEST)
         else:
-            return MessageDialog(
-                title=_('Unknown wiki type'),
-                message='',
-                action=''
-                )
+            if REQUEST:
+                return MessageDialog(
+                    title=_('Unknown wiki type'),
+                    message='',
+                    action=''
+                    )
+            else:
+                raise AttributeError, (
+                    _('The requested wiki type does not exist.'))
 
-        if REQUEST is not None:
+        if REQUEST:
             folder = getattr(self, new_id)
-            if hasattr(folder, 'setup_%s'%(wiki_type)):
+            if safe_hasattr(folder, 'setup_%s'%(wiki_type)):
                 REQUEST.RESPONSE.redirect(REQUEST['URL3']+'/'+new_id+'/setup_%s'%(wiki_type))
             elif enter:
                 # can't see why this doesn't work after addWikiFromFs
@@ -214,8 +291,10 @@ def manage_addWiki(self, new_id, new_title='', wiki_type='zwikidotorg',
                 try: u=self.DestinationURL()
                 except: u=REQUEST['URL1']
                 REQUEST.RESPONSE.redirect(u+'/manage_main?update_menu=1')
+        else: # we're called programmatically through xx.manage_addProduct...
+            return new_id
 
-def addWikiFromZodb(self,new_id, new_title='', wiki_type='zwikidotorg',
+def addWikiFromZodb(self,new_id, new_title='', wiki_type='basic',
                         REQUEST=None):
     """
     Create a new zwiki web by cloning the specified template
@@ -232,8 +311,52 @@ def addWikiFromZodb(self,new_id, new_title='', wiki_type='zwikidotorg',
     # could do stuff with ownership here
     # set it to low-privileged "nobody" by default ?
 
+def createFilesFromFsFolder(self, f, dir):
+    """
+    Recursively put the files from a fs folder into a ZODB folder.
+    """
+    filenames = os.listdir(dir)
+    for filename in filenames:
+        if re.match(r'(?:\..*|CVS|_darcs)', filename): continue
+        id, type = os.path.splitext(filename)
+        type = type.lstrip('.')
+        this_path = os.path.join(dir, filename)
+        if os.path.isdir(this_path):
+            f.manage_addFolder(filename) # add folder
+            createFilesFromFsFolder(self, f[filename], this_path) # recurse
+        else:
+            text = open(this_path, 'r').read()
+            if type == 'dtml':
+                addDTMLMethod(f, filename[:-5], title='', file=text)
+            elif re.match(r'(?:(?:rst|stx|html|latex)(?:dtml)?|txt)', type):
+                headers, body = parseHeadersBody(text)
+                if headers.has_key('title'):
+                    title = headers['title']
+                else:
+                    title = ''
+                if headers.has_key('parents'):
+                    parents = headers['parents']
+                    parents = parents.split(',')
+                else:
+                    parents = []
+                addZWikiPage(f,id,title=title,page_type=type,file=body,parents=parents)
+            elif type == 'pt':
+                f._setObject(id, ZopePageTemplate(id, text, 'text/html'))
+            elif type == 'py':
+                f._setObject(id, PythonScript(id))
+                f._getOb(id).write(text)
+            elif type == 'zexp' or type == 'xml':
+                connection = self.getPhysicalRoot()._p_jar
+                f._setObject(id, connection.importFile(dir + os.sep + filename, 
+                    customImporters=customImporters))
+                #self._getOb(id).manage_changeOwnershipType(explicit=0)
+            elif type in ['jpg','jpeg','gif','png']:
+                f._setObject(filename, Image(filename, '', text))
+            else:
+                id = f._setObject(filename, File(filename, '', text))
+                if type == 'css': f[filename].content_type = 'text/css'
 
-def addWikiFromFs(self, new_id, title='', wiki_type='zwikidotorg',
+def addWikiFromFs(self, new_id, title='', wiki_type='basic',
                       REQUEST=None):
     """
     Create a new zwiki web from the specified template on the filesystem.
@@ -245,59 +368,36 @@ def addWikiFromFs(self, new_id, title='', wiki_type='zwikidotorg',
     #str(new_id),str(title))
     # No - the standard folder's UI is more useful for small wikis,
     # and more importantly BTreeFolder2 isn't standard until 2.8.0b2
-    f = Folder()
-    f.id, f.title = str(new_id), str(title)
-    new_id = parent._setObject(f.id, f)
+    parent.manage_addFolder(str(new_id))
     f = parent[new_id]
+    f.title = str(title)
+    
     # add objects from wiki template
     # cataloging really slows this down!
     dir = os.path.join(package_home(globals()),'wikis',wiki_type)
-    filenames = os.listdir(dir)
-    for filename in filenames:
-        if re.match(r'(?:\..*|CVS|_darcs)', filename): continue
-        m = re.search(r'(.+)\.(.+)',filename)
-        id, type = filename, ''
-        if m: id, type = m.groups()
-        text = open(dir + os.sep + filename, 'r').read()
-        if type == 'dtml':
-            addDTMLMethod(f, filename[:-5], title='', file=text)
-        elif re.match(r'(?:(?:stx|html|latex)(?:dtml)?|txt)', type):
-            addZWikiPage(f,id,title='',page_type=type,file=text)
-        elif type == 'pt':
-            f._setObject(id, ZopePageTemplate(id, text, 'text/html'))
-        elif type == 'py':
-            f._setObject(id, PythonScript(id))
-            f._getOb(id).write(text)
-        elif type == 'zexp' or type == 'xml':
-            connection = self.getPhysicalRoot()._p_jar
-            f._setObject(id, connection.importFile(dir + os.sep + filename, 
-                customImporters=customImporters))
-            #self._getOb(id).manage_changeOwnershipType(explicit=0)
-        elif re.match(r'(?:jpe?g|gif|png)', type):
-            f._setObject(filename, Image(filename, '', text))
-        else:
-            id = f._setObject(filename, File(filename, '', text))
-            if type == 'css': f[filename].content_type = 'text/css'
+    createFilesFromFsFolder(self, f, dir) # recurses
     f.objectValues(spec='ZWiki Page')[0].updatecontents()
 
 def addZWikiPage(self, id, title='',
-                  page_type=PAGETYPES[0]._id, file=''):
+                  page_type=PAGETYPES[0]._id, file='', parents=[]):
     id=str(id)
     title=str(title)
 
-    # parse optional parents list
-    m = re.match(r'(?si)(^#parents:(.*?)\n)?(.*)',file)
-    if m.group(2):
-        parents = string.split(string.strip(m.group(2)),',')
+    if len(parents) == 0:
+        # parse optional parents list, the old way XXX can this be removed?
+        m = re.match(r'(?si)(^#parents:(.*?)\n)?(.*)',file)
+        if m.group(2):
+            parents = string.split(string.strip(m.group(2)),',')
+        else:
+            parents = []
+        text = m.group(3)
     else:
-        parents = []
-    text = m.group(3)
+        text = file
 
     # create zwiki page in this folder
     ob = ZWikiPage.ZWikiPage(source_string=text, __name__=id)
     ob.title = title
     ob.setPageType(page_type)
-    ob.parents = parents
 
     username = getSecurityManager().getUser().getUserName()
     ob.manage_addLocalRoles(username, ['Owner'])
@@ -306,8 +406,8 @@ def addZWikiPage(self, id, title='',
     # ownership so that the page will acquire it's owner from the parent
     # folder.
     ob._deleteOwnershipAfterAdd() # or _owner=UnownableOwner ?
-    #ob.setSubOwner('both') # regulations setup ?
     self._setObject(id, ob)
+    self[id].parents = parents # setting this earlier lost it for some pages (??)
 
     return getattr(self, id)
 
