@@ -12,7 +12,8 @@ from Globals import InitializeClass, package_home
 from Products.ZWiki.plugins import registerPlugin
 from Products.ZWiki.Defaults import registerPageMetaData
 from Products.ZWiki import Permissions
-from Products.ZWiki.Utils import BLATHER, formattedTraceback, addHook
+from Products.ZWiki.Utils import BLATHER, formattedTraceback, \
+    addHook, safe_hasattr
 from Products.ZWiki.Views import loadDtmlMethod, loadPageTemplate, TEMPLATES
      
 from Products.ZWiki.I18n import _
@@ -54,6 +55,7 @@ ISSUE_COLOURS = [
     ',open,normal   ,#ffbbbb',
     ',open,minor    ,#ffdddd',
     ',open,wishlist ,#e0e0e0',
+    ',open,funded   ,#ffffd0',
     ',open,         ,#ffe0e0',
     ',pending,      ,#ffcc77',
     ',closed,       ,#bbeebb',
@@ -79,10 +81,17 @@ class PluginTracker:
     """
     security = ClassSecurityInfo()
     
+    security.declareProtected(Permissions.View, 'hasIssues')
+    def hasIssueTracker(self): # likely.
+        """
+        Does this wiki have an issue tracker configured ?
+        """
+        return safe_hasattr(self.folder(),'issue_categories')
+
     security.declareProtected(Permissions.View, 'hasIssues')
     def hasIssues(self): # likely.
         """
-        True if this wiki has any issue pages.
+        Does this wiki have any issue pages ?
         """
         return self.issueCount() > 0
 
@@ -100,7 +109,7 @@ class PluginTracker:
         """
         return (
             self.issueNumberFrom(pagename or self.pageName()) or
-            hasattr(self.aq_base,'status')
+            safe_hasattr(self.aq_base,'status')
             ) and 1
             #hasattr(getattr(self,'aq_base',self),'status') if tests break
 
@@ -562,7 +571,7 @@ class PluginTracker:
                     self.create(page,
                                 text=open(os.path.join(dir,page+'.dtml'),'r').read(),
                                 sendmail=0,
-                                type='stx')
+                                type='html')
 
             # also, disable subtopics display under IssueTracker
             self.IssueTracker.setSubtopicsPropertyStatus(0)
